"""Module contains SPARQL templates for managing a BIBCAT instance"""
__author__ = "Jeremy Nelson, Mike Stabile"
import sys

try:
    from .ingester import PROJECT_BASE, NS_MGR
# Failed relative import
except SystemError:
    from ingester import PROJECT_BASE, NS_MGR
sys.path.append(PROJECT_BASE)
try:
    import rdfw as rdfframework
except ImportError:
    pass


NSM = NS_MGR

PREFIX = NSM.prefix()

GET_BLANK_NODE = PREFIX + """
SELECT ?subject 
WHERE {{
    ?instance <{0}> ?subject .
}}"""

GET_DIRECT_PROPS = PREFIX + """
SELECT ?dest_prop ?src_prop
WHERE {{
    ?subj kds:destClassUri <{0}> .
    ?subj kds:destPropUri ?dest_prop .
    OPTIONAL {{ ?subj kds:srcPropUri ?src_prop }} .
    OPTIONAL {{ ?subj kds:srcPropXpath ?src_prop }} .
    OPTIONAL {{ ?subj kds:srcPropKey ?src_prop }} .
}}"""

GET_LINKED_CLASSES = PREFIX + """
SELECT ?dest_prop ?dest_class ?linked_range ?subj
WHERE {{
   ?subj kds:destClassUri ?dest_class .
   ?subj kds:destPropUri ?dest_prop .
   ?subj kds:linkedRange ?linked_range .
   ?subj kds:linkedClass <{0}> .
   ?subj rdf:type kds:PropertyLinker .
}}"""


GET_ORDERED_CLASSES = PREFIX + """
SELECT ?dest_prop ?dest_class ?linked_range ?subj
WHERE {{
   ?subj kds:destClassUri ?dest_class .
   ?subj kds:destPropUri ?dest_prop .
   ?subj kds:linkedRange ?linked_range .
   ?subj kds:linkedClass <{0}> .
   ?subj rdf:type kds:OrderedPropertyLinker .
}}"""

GET_SRC_PROP = PREFIX + """
SELECT ?prop
WHERE {{
    ?subj kds:destClassUri <{0}> .
    ?subj kds:destPropUri <{1}> .
    ?subj kds:linkedClass <{2}> .
    ?subj kds:linkedRange <{3}> .
    ?subj rdf:type <{4}> .
    OPTIONAL {{ ?subj kds:srcPropUri ?prop }} .
    OPTIONAL {{ ?subj kds:srcPropXpath ?prop }} .
    OPTIONAL {{ ?subj kds:srcPropKey ?prop }} .
}}"""

HAS_MULTI_NODES = PREFIX + """
SELECT DISTINCT ?is_multi_nodes
WHERE {{
    <{0}> kds:hasIndividualNodes ?is_multi_nodes .
}}"""

DEDUP_ENTITIES = PREFIX + """
SELECT DISTINCT ?entity
WHERE {{
    ?entity <{0}> ?identifier .
    ?identifier rdf:type <{1}> .
    ?identifier rdf:value "{2}" .
}}"""

DEDUP_AGENTS = PREFIX + """
SELECT DISTINCT ?agent ?type
WHERE {{
    ?agent rdf:type ?type .
    ?agent <{0}> ?label .
    OPTIONAL {{ ?agent a bf:Person }} 
    OPTIONAL {{ ?agent a bf:Organization }} 
    filter regex(?label, "{1}")
}}"""

DEDUP_PERSON_ORG = PREFIX + """
SELECT DISTINCT ?agent ?label ?type
WHERE {
    ?agent a ?type .
    OPTIONAL { ?agent a bf:Person } .
    OPTIONAL { ?agent a bf:Organization } .
    OPTIONAL { ?agent rdfs:label ?label } .
    OPTIONAL { ?agent schema:alternativeName ?label } .
}"""

ENTITY_IRI_PATTERN = PREFIX + """
SELECT ?iri_pattern ?srcSelection ?srcFilter
WHERE {{
    ?subj rdf:type kds:IRIPattern .
    ?subj kds:destClassUri <{0}> .
    ?subj kds:iriPattern ?iri_pattern .
    ?subj <{1}> ?srcSelection .
    ?subj <{2}> ?srcFilter .
}}"""

GET_ADDL_PROPS = PREFIX + """
SELECT ?pred ?obj
WHERE {{
  <{0}> kds:destAdditionalPropUris ?subj .
  ?subj ?pred ?obj .
}}"""

GET_AGENTS = PREFIX + """
SELECT DISTINCT ?subject ?value
WHERE {{
    ?subject rdf:type <{0}> .
    ?subject <{1}> ?value .
}}"""

GET_BLANK_NODE = PREFIX + """
SELECT ?subject 
WHERE {{
    ?instance <{0}> ?subject .
}}"""

DEDUP_ENTITIES = PREFIX + """
SELECT DISTINCT ?entity
WHERE {{
    ?entity <{0}> ?identifier .
    ?identifier rdf:type <{1}> .
    ?identifier rdf:value "{2}" .
}}"""

GET_ADDL_PROPS = PREFIX + """
SELECT ?pred ?obj
WHERE {{
  <{0}> kds:destAdditionalPropUris ?subj .
  ?subj ?pred ?obj .
}}"""

GET_BLANK_NODE = PREFIX + """
SELECT ?subject 
WHERE {{
    ?instance <{0}> ?subject .
}}"""

GET_IDENTIFIERS = PREFIX + """
SELECT ?entity ?ident_value
WHERE {{
    ?entity rdf:type <{0}> .
    ?entity bf:identifiedBy ?identifier .
    ?identifier rdf:type <{1}> .
    ?identifier rdf:value ?ident_value .
}}"""

GET_ORDERED_MARC_LIST = PREFIX + """
SELECT ?marc
WHERE {{
    ?subj kds:srcOrderedPropUri/rdf:rest*/rdf:first ?marc .
    ?subj kds:destClassUri <{0}> .
}}"""





import os
import requests
import rdflib
import sys
import unittest

from unittest.mock import MagicMock

sys.path.append(os.path.abspath(os.path.curdir))
from generators.generator  import new_graph
from generators.work import WorkGenerator

class MockResponse(object):

    def __init__(self):
        self.status_code = 200
        self.output = {"results": {"bindings": []}}

    def json(self):
        return self.output



class TestEmptyWorkGenerator(unittest.TestCase):

    def setUp(self):
        requests.post = MagicMock(return_value=MockResponse())
        self.work_generator = WorkGenerator()
        self.instance_uri = self.work_generator.__generate_uri__()

    def test_init(self):
        self.assertTrue(
            isinstance(self.work_generator, 
                       WorkGenerator))
        # Default Blazegraph triplestore runs on localhost through semantic
        # server  
        self.assertEqual(self.work_generator.triplestore_url,
                         "http://localhost:9999/blazegraph/sparql")

    def test__add_creators__(self):
        self.work_generator.__add_creators__(new_graph(), None, self.instance_uri)

    #def test__add_creators__

    def test__work_title__(self):
        self.work_generator.__add_work_title__(new_graph(), None, self.instance_uri)

    def test__raise_assert_error_copy_instance_to_work__(self):
        with self.assertRaises(AssertionError):
            self.work_generator.__copy_instance_to_work__(self.instance_uri, 
                None)

    def test__raise_error_copy_instance_to_work__(self):
        with self.assertRaises(ValueError):
            self.work_generator.__copy_instance_to_work__(
                self.instance_uri, 
                self.instance_uri)

    def test__generate_work__(self):
        self.assertIsNotNone(
            self.work_generator.__generate_work__(
                self.instance_uri))

    def test__similiar_creators__(self):
        self.work_generator.__similar_creators__(
            str(self.instance_uri))
        self.assertEqual(
            self.work_generator.matched_works,
            [])


    def test__similiar_titles__(self):
        self.work_generator.__similar_titles__(str(self.instance_uri))
        self.assertEqual(
            self.work_generator.matched_works,
            [])



    def test_harvest_instances(self):
        self.work_generator.harvest_instances()
        self.assertEqual(
            self.work_generator.matched_works,
            [])


    def test_run(self):
        self.work_generator.run()
        self.assertEqual(
            self.work_generator.matched_works,
            [])

class TestSuccessWorkGenerator(unittest.TestCase):

    def setUp(self):
        self.found_work = rdflib.URIRef("http://bibcat.org/TestWork-1234")
        self.work_generator = WorkGenerator()

    def test_run(self):
        found_response = MockResponse()
        found_response.output['results']['bindings'].append(
            {"instance": {"value": str(self.found_work)}})
        requests.post = MagicMock(return_value=found_response)
        self.work_generator.run()

        self.assertEqual(
            self.work_generator.matched_works,
            [])
        


if __name__ == '__main__':
    unittest.main()
        



__author__ = "Jeremy Nelson"

import os
import rdflib
import sys
import unittest
import urllib.parse

sys.path.append(os.path.abspath(os.path.curdir))
import linkers


class TestLinker(unittest.TestCase):

    def setUp(self):
        self.linker = linkers.Linker()

    def test_init(self):
        self.assertIsInstance(self.linker, linkers.Linker)

    def test_run(self):
        self.assertTrue(hasattr(self.linker, 'run'))





if __name__ == "__main__":
    unittest.main()



"""BIBCAT is a RDF-based Bibliographic Catalog"""
import re
import urllib.parse
import pkg_resources
import rdflib
from rdflib.term import _is_valid_uri

__author__ = "Jeremy Nelson, Mike Stabile, Jay Peterson"
__version__ = pkg_resources.get_distribution("bibcat").version

def clean_uris(graph):
    """Iterates through all URIRef subjects and objects and attempts to fix any
    issues with URL.

    Args:
        graph(rdflib.Graph): BIBFRAME RDF Graph
    """
    def fix_uri(uri):
       url_sections = urllib.parse.urlparse(str(uri))
       new_url = (url_sections.scheme,
                  url_sections.netloc,
                  urllib.parse.quote(url_sections.path),
                  urllib.parse.quote(url_sections.params),
                  urllib.parse.quote(url_sections.query),
                  urllib.parse.quote(url_sections.fragment))
       new_uri = rdflib.URIRef(
           str(urllib.parse.urlunparse(new_url)))
       for pred, obj in graph.predicate_objects(subject=uri):
           graph.remove((uri, pred, obj))
           graph.add((new_uri, pred, obj)) 
       for subj, pred in graph.subject_predicates(object=uri):
           graph.remove((subj, pred, uri))
           graph.add((subj, pred, new_uri))
    ALL_URI_SPARQL = """SELECT DISTINCT ?uri
        WHERE {
            ?uri ?p ?o .
            ?s ?p1 ?uri .
        FILTER(isIRI(?uri))
    }"""
    for iri in graph.query(ALL_URI_SPARQL):
         try:
             if _is_valid_uri(str(iri[0])) is False:
                fix_uri(iri[0])
         except rdflib.exceptions.SubjectTypeError:
             fix_uri(iri)

def replace_iri(graph, old_iri, new_iri):
    """Replaces old IRI with a new IRI in the graph

    Args:

    ----
        graph: rdflib.Graph
        old_iri: rdflib.URIRef, Old IRI
        new_iri: rdflib.URIRef, New IRI
    """
    for pred, obj in graph.predicate_objects(subject=old_iri):
        graph.add((new_iri, pred, obj))
        graph.remove((old_iri, pred, obj))
    for subj, pred in graph.subject_predicates(object=old_iri):
        graph.add((subj, pred, new_iri))
        graph.remove((subj, pred, old_iri))


def slugify(value):
    """
    Converts to lowercase, removes non-word characters (alphanumerics and
    underscores) and converts spaces to hyphens. Also strips leading and
    trailing whitespace. Adapted from Django's slugify function
    """
    value = re.sub(r'[^\w\s-]', '', value).strip().lower()
    return re.sub(r'[-\s]+', '-', value)


def wikify(value):
    """Converts value to wikipedia "style" of URLS, removes non-word characters
    and converts spaces to hyphens and leaves case of value.
    """
    value = re.sub(r'[^\w\s-]', '', value).strip()
    return re.sub(r'[-\s]+', '_', value)






__author__ = "Jeremy Nelson, Mike Stabile"

import logging
import os
import sys
import unittest
sys.path.append(os.path.abspath(os.path.curdir))
BIBCAT_BASE = os.path.abspath(
    os.path.split(
        os.path.dirname(__file__))[0])
PROJECT_BASE = os.path.split(BIBCAT_BASE)[0]
sys.path.append(PROJECT_BASE)
try:
    from instance import config
except ModuleNotFoundError:
    class Config(object):
        def __init__(self):
            self.BASE_URL = "http://bibcat.org/"
            self.TRIPLESTORE_URL = "http://localhost:9999/blazegraph/sparql"
             
    config = Config()

import ingesters
from ingesters.ingester import NS_MGR
from ingesters.csv import RowIngester

ingesters.MLOG_LVL = logging.CRITICAL
logging.getLogger("requests").setLevel(logging.CRITICAL)
logging.getLogger("urllib3").setLevel(logging.CRITICAL)


class TestRowIngester(unittest.TestCase):

    def setUp(self):
        self.ingester = RowIngester()
        

    def test_init(self):
        self.assertIsInstance(self.ingester, RowIngester)

    def test_defaults(self):
        if hasattr(config, 'BASE_URL'):
            self.assertEqual(
                self.ingester.base_url,
                config.BASE_URL)
        self.assertEqual(
            len(self.ingester.graph),
            0)
        self.assertTrue(
            len(self.ingester.rules_graph) < 1)
        self.assertIsNone(self.ingester.source)
        self.assertIn(
            self.ingester.triplestore_url,
            ['http://localhost:9999/blazegraph/sparql',
             config.TRIPLESTORE_URL])


if __name__ == "__main__":
   unittest.main() 



import unittest
import os
import sys
import unittest
sys.path.append(os.path.abspath(os.path.curdir))
from generators.generator import Generator, config

class TestGenerator(unittest.TestCase):

    def setUp(self):
        del(config.BASE_URL)
        self.generator = Generator()

    def test_defaults(self):
        self.assertEqual(
            self.generator.triplestore_url,
            "http://localhost:9999/blazegraph/sparql")
        self.assertEqual(
            self.generator.base_url,
            "http://bibcat.org/")

if __name__ == '__main__':
    unittest.main()



__author__ = "Jeremy Nelson"

import os
import rdflib
import sys
import unittest
import urllib.parse

sys.path.append(os.path.abspath(os.path.curdir))
from linkers import NS_MGR
import linkers.loc as loc

class TestLibraryOfCongressLinker(unittest.TestCase):

    def setUp(self):
        self.linker = loc.LibraryOfCongressLinker()

    def test_default_init(self):
        self.assertTrue(self.linker, loc.LibraryOfCongressLinker)
        self.assertEqual(
            self.linker.ID_LOC_URL,
            "http://id.loc.gov/")

if __name__ == "__main__":
    unittest.main()




"""Helper module for linking existing BIBFRAME resources to external data
sources like Library of Congress, DBPedia, VIAF, and others."""

__author__ = "Jeremy Nelson, Mike Stabile"

import os
import rdflib
import sys
BIBCAT_BASE = os.path.abspath(
    os.path.split(
        os.path.dirname(__file__))[0])

sys.path.append(BIBCAT_BASE)
from ingesters.ingester import new_graph, NS_MGR, PROJECT_BASE
sys.path.append(PROJECT_BASE)
try:
    import rdfw as rdfframework
except ImportError:
    pass
from rdfframework.utilities import RdfNsManager

class Linker(object):
    """Base Linker class for all other linker classes"""
    NS = NS_MGR

    def __init__(self, **kwargs):
        self.triplestore_url = kwargs.get(
            "triplestore_url",
            "http://localhost:9999/blazegraph/sparql")


    def run(self):
        pass

class LinkerError(Exception):
    """Custom Error for Linker Classes"""

    def __init__(self, value, details):
        self.value = value
        self.details = details

    def __str__(self):
        return repr(self.value)



"""Fedora 3.x RELS-EXTseries to BIBFRAME 2.0 ingester

This ingester is not intended to generated fully formed BF RDF but 
supplement existing ingesters like MODS and DC. The RELS-EXT ingester adds
additional properties and classes to existing BF entities. 

"""
__author__ = "Jeremy Nelson, Mike Stabile"

import os
import rdflib
import requests

from bibcat.rml.processor import XMLProcessor

class RELSEXTIngester(XMLProcessor):

    def __init__(self, **kwargs):
        rules = ["bibcat-rels-ext.ttl"]
        if "rules_ttl" in kwargs:
            tmp_rules = kwargs.get("rules_ttl")
            if isinstance(tmp_rules, str):
                rules.append(tmp_rules)
            elif isinstance(tmp_rules, list):
                rules.extend(tmp_rules)
        super(RELSEXTIngester, self).__init__(
            rml_rules=rules,
            base_url=kwargs.get("base_url", "http://bibcat.org/"),
            institution_iri=kwargs.get("institution_iri"),
            namespaces={'fedora': 'info:fedora/fedora-system:def/relations-external#',
                'fedora-model': 'info:fedora/fedora-system:def/model#',
                'islandora': 'http://islandora.ca/ontology/relsext#',
                'rdf': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'})



"""Helper Class and Functions for resolving Library Linked Data Carriers"""
__author__ = "Jeremy Nelson, Mike Stabile"

import os
from linker import Linker, LinkerError, PROJECT_BASE, new_graph 

CARRIERS = new_graph()
ttl_file_path =os.path.join(
    os.path.join(PROJECT_BASE, "bibcat", "rdf-references"),
    "bc_carriers.ttl")
with open(ttl_file_path) as fo:
    CARRIERS.parse(fo, format='turtle') 


class CarrierLinker(Linker):
    """Links existing Library of Congress Carrier Types to BF Instance"""

    def __init__(self, **kwargs):
        super(CarrierLinker, self).__init__(**kwargs)


    def run(self):
        """Method runs the linker on existing triplestore"""
        result = requests.post(self.triplestore_url,
            data={"query": CARRIER_SPARQL,
                  "format": "json"})
        if result.status_code > 399:
            raise LinkerError("CARRIER_SPARQL failed on {}".format(
                self.triplestore_url),
                "HTTP status={}\n{}".format(result.status_code,
                                            result.text))

        bindings 
        
        
CARRIER_SPARQL = Linker.NS.prefix() + """

SELECT ?instance ?value
WHERE {
    ?instance bf:carrier ?carrier .
    ?carrier rdf:type bf:Carrier .
    ?carrier rdf:value ?value 
    filter isBlank(?carrier)
}"""



__author__ = "Jeremy Nelson"

import os
import rdflib
import sys
import unittest
import urllib.parse

sys.path.append(os.path.abspath(os.path.curdir))
from linkers import NS_MGR
import linkers.dbpedia as dbpedia


class TestDBPediaLinker(unittest.TestCase):

    def setUp(self):
        self.linker = dbpedia.DBPediaLinker()


    def test_enhance_uri_default(self):
        test_uri = rdflib.URIRef("http://bibcat.org/sample-uri")
        enhanced_graph = self.linker.enhance_uri(
            test_uri, 
            "http://dbpedia.org/resource/Moby-Dick")
        self.assertTrue(isinstance(enhanced_graph, rdflib.Graph))

    def test_search_label_default(self):
        moby_dick_results = self.linker.search_label("Moby Dick")
        self.assertEqual(
            len(moby_dick_results),
            7)
    def test_search_label_book(self):
        moby_dick_results = self.linker.search_label(
            "Moby Dick", 
            [NS_MGR.dbo.Book,])
        self.assertEqual(
            moby_dick_results[0].get('value'),
            "http://dbpedia.org/resource/Moby-Dick")

    def test_search_label_film(self):
        moby_dick_results = self.linker.search_label(
            "Moby Dick", 
            [NS_MGR.dbo.Film,])
        self.assertIn(
            {"dbo:class": NS_MGR.dbo.Film,
             "type": "uri",
             "value": "http://dbpedia.org/resource/Moby_Dick_(1978_film)"},
            moby_dick_results)

    def test_search_label_musical_work(self):
        john_wesley_harding_results = self.linker.search_label(
            "John Wesley Harding",
            [NS_MGR.dbo.MusicalWork,])
        self.assertIn(
            {"dbo:class": NS_MGR.dbo.MusicalWork,
             "type": "uri",
             "value": "http://dbpedia.org/resource/John_Wesley_Harding_(album)"},
            john_wesley_harding_results)


    def test_default_init(self):
        self.assertIsInstance(self.linker, dbpedia.DBPediaLinker)
        self.assertEqual(
            self.linker.SPARQL_ENDPOINT,
            "http://dbpedia.org/sparql")

if __name__ == "__main__":
    unittest.main()



"""Helper Class and Functions for linking BIBFRAME 2.0 linked data with 
Library of Congress id.loc.gov linked-data webservices"""
__author__ = "Jeremy Nelson, Mike Stabile"

import datetime
import os
import requests
import rdflib
try:
    from ..ingesters.ingester import new_graph, NS_MGR
    from .linker import BIBCAT_BASE, Linker, LinkerError
except ValueError:
    from ingesters.ingester import new_graph, NS_MGR
    from linkers.linker import BIBCAT_BASE, Linker, LinkerError


from fuzzywuzzy import fuzz

LCSH_GRAPH = new_graph()
LCSH_GRAPH.parse(os.path.join(BIBCAT_BASE, "rdf-references/loc_subjects.ttl"),
                 format='turtle')

class LibraryOfCongressLinker(Linker):
    """Library of Congress Linked Data Linker"""
    ID_LOC_URL = "http://id.loc.gov/"

    def __init__(self, **kwargs):
        super(LibraryOfCongressLinker, self).__init__(**kwargs)


    def __link_lcsh__(self, instance_uri, label, cutoff=90):
        """Attempts to match incoming bnode subject label to LCSH iri"""
        for row in LCSH_GRAPH.query(FIND_LCSH.format(label)):
            subject_uri, subject_label = row
            if fuzz.ratio(subject_label, label) >= cutoff:
                update_sparql = UPDATE_SUBJECT.format(
                                    subject_uri,
                                    instance_uri, 
                                    label)
                result = requests.post(self.triplestore_url,
                            data=update_sparql,
                            headers={"Content-Type": "application/sparql-update"})
                if result.status_code > 399:
                    raise LinkerError(
                        "Failed bf:subject with LCSH iri http error code={}".format(
                            result.status_code,
                            result.text))
       

    def run(self):
        """Runs linker on existing bf:subject Blank Nodes"""
        result = requests.post(self.triplestore_url,
            data={"query": SUBJECT_BNODES,
                  "format": "json"})
        if result.status_code > 399:
            raise LinkerError("Failed to run SUBJECT_BNODES sparql query {}".format(
                    result.status_code),
                result.text)
        bindings = result.json().get('results').get('bindings')
        start = datetime.datetime.utcnow()
        print("Starting LCSH Linker Service at {}, total to process {}".format(
            start,
            len(bindings)))
        for i,row in enumerate(bindings):
            instance_uri = row.get('instance').get('value')
            label = row.get('label').get('value')
            self.__link_lcsh__(instance_uri, label)
            if not i%10 and i > 0:
                print(".", end="")
            if not i%100:
                print(i, end="")
        end = datetime.datetime.utcnow()
        print("Finished LCSH Linker Service at {}, total time={} mins".format(
            end,
            (end-start).seconds /60.0))

FIND_LCSH = Linker.NS.prefix() + """

SELECT *
{{
  OPTIONAL {{
     ?1 rdfs:label "{0}"@en
   }}
   OPTIONAL {{
     ?2 rdfs:label ?lab
     filter(CONTAINS(?lab, "{0}"))
   }}
}}
"""

SUBJECT_BNODES = Linker.NS.prefix() + """

SELECT ?instance ?label 
WHERE {
    ?instance bf:subject ?subject .
    ?subject rdf:value ?label .
    filter(isblank(?subject))
}"""

UPDATE_SUBJECT = Linker.NS.prefix() + """

DELETE {{
   ?instance bf:subject ?sub_bnode .
   ?sub_bnode ?p ?o .
}} INSERT {{
    BIND(<{0}> as ?lcsh)
    ?instance bf:subject ?lcsh .
}} WHERE {{
    BIND(<{1}> as ?instance)
    ?instance bf:subject ?sub_bnode .
    ?sub_bnode rdfs:label "{2}" .
}}"""



"""OAI-PMH to BIBFRAME 2.0 ingester Classes"""
__author__ = "Jeremy Nelson, Mike Stabile"

import click
import datetime
import io
import logging
import os
try:
    from lxml import etree
except ImportError:
    import xml.etree.ElementTree as etree
import rdflib
import requests
import sys
import urllib.parse
import uuid

from .ingester import new_graph, NS_MGR, BIBCAT_BASE
from .rels_ext import RELSEXTIngester
from ..rml.processor import XMLProcessor

NS = {"oai_pmh": "http://www.openarchives.org/OAI/2.0/",
      'dc': 'http://purl.org/dc/elements/1.1/', 
      'oai_dc': 'http://www.openarchives.org/OAI/2.0/oai_dc/'}
try:
    NS_MGR.bind('fedora', 'info:fedora/fedora-system:def/relations-external#')
    NS_MGR.bind('fedora-model', 'info:fedora/fedora-system:def/model#')
except AttributeError:
    setattr(NS_MGR, 'fedora', 'info:fedora/fedora-system:def/relations-external#')
    setattr(NS_MGR, 'fedora-model', 'info:fedora/fedora-system:def/model#')

class OAIPMHIngester(object):
    IDENT_XPATH = "oai_pmh:ListIdentifiers/oai_pmh:header/oai_pmh:identifier"
    TOKEN_XPATH = "oai_pmh:ListIdentifiers/oai_pmh:resumptionToken"

    def __init__(self, **kwargs):
        self.oai_pmh_url = kwargs.get("repository")
        if self.oai_pmh_url is None:
            raise ValueError("repository must have a value")
        self.identifiers = dict()
        self.metadataPrefix = "oai_dc"
        metadata_url = "{}?verb=ListMetadataFormats".format(self.oai_pmh_url)
        metadata_result = requests.get(metadata_url)
        metadata_formats = metadata_result.text
        if isinstance(metadata_result.text, str):
            metadata_formats = metadata_result.text.encode()
        self.metadata_formats_doc = etree.XML(metadata_formats)
        self.processor = None
    
    def harvest(self, **kwargs):
        """Method harvests all identifiers using ListIdentifiers"""
        params = {"verb": "ListIdentifiers",
                  "metadataPrefix": self.metadataPrefix}
        if "setSpec" in kwargs:
            params["set"] = kwargs.get("setSpec")
        initial_url = "{0}?{1}".format(
            self.oai_pmh_url,
            urllib.parse.urlencode(params))
        initial_result = requests.get(initial_url)
        if initial_result.status_code > 399:
            raise ValueError("Cannot Harvest {}, result {}".format(
                self.oai_pmh_url,
                initial_result.text))
        raw_initial = initial_result.text
        if isinstance(raw_initial, str):
            raw_initial = raw_initial.encode()
        initial_doc = etree.XML(raw_initial)
        resume_token = initial_doc.find(OAIPMHIngester.TOKEN_XPATH, NS)
        for r in initial_doc.findall(OAIPMHIngester.IDENT_XPATH, NS):
            ident = r.text
            if not ident in self.identifiers:
                self.identifiers[ident] = 1
        if resume_token is not None:
            total_size = int(resume_token.attrib.get("completeListSize", 0))
        else:
            total_size = 0
        sample_size = kwargs.get('sample_size')
        start = datetime.datetime.utcnow()
        msg = "Started Retrieval of {} Identifiers {}".format(total_size, start)
        try:
            click.echo(msg)
        except io.UnsupportedOperation:
            print(msg)
        while len(self.identifiers) < total_size:
            continue_url = "{0}?verb=ListIdentifiers&resumptionToken={1}".format(
                self.oai_pmh_url,
                resume_token.text)
            result = requests.get(continue_url)
            shard_raw = result.text
            if isinstance(shard_raw, str):
                shard_raw = shard_raw.encode()
            shard_doc = etree.XML(shard_raw)
            resume_token = shard_doc.find(OAIPMHIngester.TOKEN_XPATH, NS)
            for r in shard_doc.findall(OAIPMHIngester.IDENT_XPATH, NS):
                if not r.text in self.identifiers:
                    self.identifiers[r.text] = 1
            try:
                click.echo(".", nl=False)
            except io.UnsupportedOperation:
                print(".", end="")
        # Creates a random sample of identifiers of sample_size length
        if sample_size is not None:
            import random
            rand_keys = []
            for i in range(int(sample_size)):
                rand_keys.append(random.randint(0, total_size))
            for i, key in enumerate(list(self.identifiers.keys())):
                if not i in rand_keys:
                    self.identifiers.pop(key)
        msg = "Sample size {} identifiers size {}".format(sample_size, len(self.identifiers))
        try:
            click.echo(msg)
        except io.UnsupportedOperation:
            print(msg)        
        end = datetime.datetime.utcnow()
        msg = "\nFinished at {}, total time {} minutes".format(
            end,
            (end-start).seconds / 60.0)
        try:
            click.echo(msg)
        except io.UnsupportedOperation:
            print(msg)


class ContentDMIngester(OAIPMHIngester):
    """ContentDM Ingester provides an interface to OCLC's ContentDM&copy; 
    repository"""

    def __init__(self, **kwargs):
        super(ContentDMIngester, self).__init__(**kwargs)
        self.triplestore_url = kwargs.get("triplestore_url")
        rml_rules = kwargs.get("rml_rules", [])
        if not isinstance(rml_rules, list):
            rml_rules = [rml_rules,]
        # Add rml base and OAI-PMH DC rules
        for rulefile in ["bibcat-oai-pmh-dc-xml-to-bf.ttl",
                         "bibcat-base.ttl"]:
            rml_rules.append(rulefile)
        self.processor = XMLProcessor(
            institution_iri=kwargs.get("institution_iri"),
            instance_iri = kwargs.get('instance_iri'),
            rml_rules=rml_rules,
            namespaces=NS)

    def __process_dc__(self, **kwargs):
        """Method processes Dublin Core RDF"""

    def harvest(self, **kwargs):
        """Method harvests either the entire repository contents or selected
        collections"""
        start = datetime.datetime.utcnow()
        msg = "Starting OAI-PMH harvest of PIDS from ContentDM at {}".format(
            start)
        try:
            click.echo(msg)
        except io.UnsupportedOperation:
            print(msg)
        params = {"verb": "ListRecords",
                  "metadataPrefix": self.metadataPrefix}
        if "setSpec" in kwargs:
            params["set"] = kwargs.get("setSpec")
        initial_result = requests.get("{0}?{1}".format(
                self.oai_pmh_url,
                urllib.parse.urlencode(params)))
        initial_doc = etree.XML(initial_result.text.encode())
        token = initial_doc.find("oai_pmh:ListRecords/oai_pmh:resumptionToken", NS)
        records = initial_doc.findall("oai_pmh:ListRecords/oai_pmh:record", NS)
        self.repo_graph = rdflib.Graph()#self.processor.__graph__()
        count = 0
        for i,rec in enumerate(records):
            self.processor.run(rec, **kwargs)
            self.repo_graph += self.processor.output
           
            if not i%10 and i > 0:
                try:
                    click.echo(".", nl=False)
                except io.UnsupportedOperation:
                     print(".", end="")
            if not i%100:
                try:
                    click.echo(i, nl=False)
                except io.UnsupportedOperation:
                     print(i, end="")
        return 
        count = i
        while token is not None:
            params = { "resumptionToken": token.text,
                       "verb": "ListRecords"}
            continue_url = "{0}?{1}".format(
                self.oai_pmh_url,
                urllib.parse.urlencode(params))
            shard_result = requests.get(continue_url)
            shard_doc = etree.XML(shard_result.text.encode())
            token = shard_doc.find("oai_pmh:ListRecords/oai_pmh:resumptionToken", NS)
            records = shard_doc.findall("oai_pmh:ListRecords/oai_pmh:record", NS)
            for rec in records:
                self.processor.run(rec, **kwargs)
                self.repo_graph += self.processor.output
                if not count%10 and count > 0:
                    try:
                        click.echo(".", nl=False)
                    except io.UnsupportedOperation:
                         print(".", end="")
                if not count%100:
                    try:
                        click.echo(count, nl=False)
                    except io.UnsupportedOperation:
                         print(count, end="")
                count += 1
                end = datetime.datetime.utcnow()
            if count > 8000:
                print(token.text, count)
        msg = "\nContentDM OAI-PMH harvested at {}, total time {} mins".format(
            end,
            (end-start).seconds / 60.0)
        try:
            click.echo(msg)
        except io.UnsupportedOperation:
            print(msg)       
        if 'out_file' in kwargs:
            self.repo_graph += self.processor.output
        else:
            self.add_to_triplestore()



class IslandoraIngester(OAIPMHIngester):
    """Islandora Ingester brings together multiple ingesters to deal with MODS and 
    RELS-EXT Metadata in order to generate BIBFRAME RDF"""
    MODS_XPATH = "oai_pmh:ListMetadataFormats/oai_pmh:metadataFormat[oai_pmh:metadataPrefix='mods']"

    def __init__(self, **kwargs):
        if not kwargs['repository'].endswith("oai2"):
            kwargs['repository'] = urllib.parse.urljoin(
                kwargs['repository'], 
                "oai2")
        self.repository = kwargs.get('repository')
        super(IslandoraIngester, self).__init__(**kwargs)
        self.repo_graph = rdflib.Graph()
        self.repo_graph.namespace_manager.bind("bf", "http://id.loc.gov/ontologies/bibframe/")
        self.repo_graph.namespace_manager.bind("relators", "http://id.loc.gov/vocabulary/relators/")
        self.base_url = kwargs.get('base_url')
        rules_ttl = kwargs.get("rules_ttl", [])
        if self.metadata_formats_doc.find(
            IslandoraIngester.MODS_XPATH, 
            NS) is not None:
            self.metadataPrefix = "mods"
            for rule_name in ["bibcat-base.ttl", 
                              "bibcat-mods-to-bf.ttl"]:
                rules_ttl.append(rule_name)
            self.processor = XMLProcessor(
                rml_rules=rules_ttl,
                base_url=self.base_url,
                triplestore_url=kwargs.get("triplestore_url"),
                institution_iri=kwargs.get("institution_iri"),
                namespaces={self.metadataPrefix: str(NS_MGR.mods)})
        else:
            rules_ttl.append(
                os.path.join(BIBCAT_BASE,
                    os.path.join("rdfw-definitions", "bibcat-base.ttl")))
            self.processor = XMLProcessor(
                rml_rules=rules_ttl)


    def __process_mods__(self, **kwargs):
        """Extracts MODS datastream from the item_url

        keyword args:
            item_url(str): URL For Fedora URL
        """
        item_url = kwargs.get('item_url')
        mods_url = urllib.parse.urljoin(item_url,
            "datastream/MODS")
        mods_result = requests.get(mods_url)
        base_url = kwargs.get("base_url")
        if base_url is None and "base_url" in self.processor.constants:
            base_url = self.processor.constants.get("base_url")
        else:
            raise ValueError("base_url required for __process_mods__")
        instance_url = kwargs.get("instance_url")
        if instance_url is None:
            instance_url = "{0}/{1}".format(base_url, uuid.uuid1()) 
        try:
            self.processor.run(mods_result.text,
                base_url=base_url,
                id=uuid.uuid1,
                item_iri=item_url,
                instance_iri=instance_url)
        except:
            logging.error("{} Error with {}".format(
                sys.exc_info()[1],
                item_url))

    def __process_rels_ext__(self, **kwargs):
        """Extracts RELS-EXT and returns RELS-EXT ingester"""
        item_url = kwargs.get('item_url')
        rels_ext_url = urllib.parse.urljoin(item_url,
            "datastream/RELS-EXT")
        rels_ext_result = requests.get(rels_ext_url)
        if rels_ext_result.status_code > 399:
            error = "{} RELS-EXT not found".format(item_url)
            try:
                 click.echo(error, nl=False)
            except io.UnsupportedOperation:
                 print(error, end=" ")
            return None, None
        base_url = kwargs.get("base_url")
        if base_url is None and self.base_url is not None:
            base_url = base_url
        else:
            raise ValueError("base_url required for __process_rels_ext__") 
        rels_ext = RELSEXTIngester(base_url=base_url)
        rels_ext_doc = etree.XML(rels_ext_result.text)
        # Returns None if object is part of a Compound Object
        if rels_ext_doc.find("rdf:Description/fedora:isConstituentOf",
            rels_ext.xml_ns) is not None:
            return None, None
        return rels_ext, rels_ext_doc


    def harvest(self, **kwargs):
        """Overloaded harvest method takes optional RELS-EXT ttl file"""
        start = datetime.datetime.utcnow()
        msg = "Starting OAI-PMH harvest of PIDS from Islandora at {}".format(
            start)
        try:
            click.echo(msg)
        except io.UnsupportedOperation:
            print(msg)
        sample_size = kwargs.get('sample_size')
        super(IslandoraIngester, self).harvest(
            sample_size=sample_size,
            setSpec=kwargs.get('setSpec'))
        deduplicator = kwargs.get('dedup')
        for i,row in enumerate(self.identifiers.keys()):
            if not i%10 and i > 0:
                try:
                    click.echo(".", nl=False)
                except io.UnsupportedOperation:
                    print(".", end="")
            if not i%100:
                try:
                    click.echo(i, nl=False)
                except io.UnsupportedOperation:
                    print(i, end="")
            pid = row.split(":")[-1].replace("_", ":")
            item_url = urllib.parse.urljoin(self.repository,
                "islandora/object/{0}/".format(pid))
            item_uri = rdflib.URIRef(item_url)
            rels_ext, rels_ext_doc = self.__process_rels_ext__(
                item_url=item_url)
            if rels_ext is None:
                continue
            self.__process_mods__(item_url=item_url)
            instance_uri = self.processor.output.value(
                subject=item_uri,
                predicate=NS_MGR.bf.itemOf)
            rels_ext.run(rels_ext_doc,
                instance_iri=instance_uri)
            self.processor.output += rels_ext.output
            if deduplicator:
                deduplicator.run(self.processor.output,
                    kwargs.get("dedup_classes"))
            if 'out_file' in kwargs:
                self.repo_graph += self.processor.output
            else:
                self.processor.add_to_triplestore()
        if 'out_file' in kwargs:
            with open(kwargs.get('out_file'), 'wb+') as fo:
                fo.write(self.repo_graph.serialize(format='turtle'))
        end = datetime.datetime.utcnow()
        msg = "\nIslandora OAI-PMH harvested at {}, total time {} mins".format(
            end,
            (end-start).seconds / 60.0)
        try:
            click.echo(msg)
        except io.UnsupportedOperation:
            print(msg)

class LunaIngeseter(OAIPMHIngester):
    """Harvests Luna objects from a OAI-PMH feed"""
    
    def __init__(self, **kwargs):
        super(LunaIngeseter, self).__init__(**kwargs)




__author__ = "Jeremy Nelson"

import sys
import unittest

from bibcat.rml.processor import JSONProcessor

class TestJSONProcessorInit(unittest.TestCase):

    def setUp(self):
        pass

    def test_default_exceptions(self):
        self.assertRaises(Exception, JSONProcessor)
        # Windows error
        if sys.platform.startswith("win"):
            self.assertRaises(PermissionError, JSONProcessor, rml_rules="")
        else:
            self.assertRaises(IsADirectoryError, JSONProcessor, rml_rules="")
        self.assertRaises(Exception, JSONProcessor, rml_rules=[])

    def test_default_base(self):
        processor = JSONProcessor(rml_rules=["bibcat-base.ttl"])
        self.assertEqual(len(processor.rml), 16)

        

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



"""BIBFRAME 2.0 Generator"""
import logging
import os
import rdflib
import sys
import uuid
__author__ = "Jeremy Nelson, Mike Stabile"

BIBCAT_BASE = os.path.abspath(
    os.path.split(
        os.path.dirname(__file__))[0])
PROJECT_BASE = os.path.split(BIBCAT_BASE)[0]
sys.path.append(PROJECT_BASE)
try:
    from instance import config
    import rdfw as rdfframework
    from rdfframework.utilities import RdfNsManager
    RdfNsManager.log_level = logging.CRITICAL
except ImportError:
    logging.error("Error importing {}".format(PROJECT_BASE))
try:
    version_path = os.path.join(
        BIBCAT_BASE,
        "VERSION")
    with open(version_path) as version:
        __version__ = version.read().strip()
except:
    __version__ = "unknown"

NS_MGR = RdfNsManager()
NS_MGR.bind("bf", "http://id.loc.gov/ontologies/bibframe/")
NS_MGR.bind("kds", "http://knowledgelinks.io/ns/data-structures/")
NS_MGR.bind("schema", "http://schema.org/")
NS_MGR.bind("owl", rdflib.OWL)
NS_MGR.bind("relators", "http://id.loc.gov/vocabulary/relators/")

class Generator(object):
    """Base class for all BIBFRAME generators"""

    def __init__(self, **kwargs):
        """Creates defaults for all generators"""
        self.triplestore_url = kwargs.get(
            "url",
            "http://localhost:9999/blazegraph/sparql")
        self.base_url = kwargs.get("base_url")
        if self.base_url is None:
            if hasattr(config, "BASE_URL"):
                self.base_url = config.BASE_URL
            else:
                self.base_url = "http://bibcat.org/"

    def __generate_uri__(self):
        """Method generates an URI based on the base_url"""
        uid = uuid.uuid1()
        if self.base_url.endswith("/"):
            pattern = "{0}{1}"
        else:
            pattern = "{0}/{1}"
        return rdflib.URIRef(pattern.format(self.base_url, uid))

    def report(self, start, end):
        """Runs a report on the generator, should be overridden by children
        classes"""
        pass

    def run(self):
        """Runs generator on triplestore, method should be overridden by
        children classes"""
        pass

def new_graph():
    """Function creates a new graph with RDF Framework namespace
    Manager"""
    graph = rdflib.Graph(namespace_manager=NS_MGR)
    return graph



__author__ = "Jeremy Nelson"

import os
import rdflib
import sys
import unittest
from unittest import mock
from types import SimpleNamespace
try:
    import bibcat.rml.processor as processor
    import bibcat    
except ImportError:
    sys.path.append(os.path.abspath("."))
    import bibcat.rml.processor as processor
    import bibcat

TESTS_PATH = os.path.dirname(os.path.abspath(__file__))
FIXURES_PATH = os.path.join(
    TESTS_PATH, 
    "fixures")

class TestRDFMappingLanguageProcessor(unittest.TestCase):

    def setUp(self):
        pass

    def test_defaults(self):
        self.assertRaises(TypeError, processor.Processor)
     
    def test_bibcat_base(self):
        base_processor = processor.Processor(
            rml_rules=os.path.join(FIXURES_PATH,
                                   "rml-basic.ttl"))
        self.assertTrue(isinstance(base_processor.rml, rdflib.Graph))

    def test_bibcat_rules_missing_rml_rules_error(self):
        self.assertRaises(TypeError, processor.Processor, rml_rules=None)
        # Windows error
        if sys.platform.startswith("win"):
            self.assertRaises(PermissionError, processor.Processor, rml_rules="")
        else:
            self.assertRaises(IsADirectoryError, processor.Processor, rml_rules="")
        self.assertRaises(Exception, processor.Processor, rml_rules=[])


    def test_bibcat_rules_blank_graphs_rml_rules_error(self):
        self.assertRaises(Exception, processor.Processor, rml_rules=rdflib.Graph())
        self.assertRaises(Exception, 
            processor.Processor, 
            rml_rules=rdflib.ConjunctiveGraph())
        

    def test_bibcat_acceptable_path_rules(self):
        raw_rml = os.path.join(FIXURES_PATH,
                               "rml-basic.ttl")
        self.assertTrue(os.path.exists(raw_rml))
        self.assertTrue(
            isinstance(processor.Processor(raw_rml),
                       processor.Processor))
                      
    def test_bibcat_package_rule(self):
        self.assertTrue(
            isinstance(processor.Processor("bibcat-base.ttl"),
                      processor.Processor))
 
    def test_bibcat_acceptable_list_rules(self):
        rules = []
        for name in ["rml-basic.ttl", "rml-basic.ttl"]:
             rules.append(os.path.join(FIXURES_PATH, name))
        self.assertTrue(
            isinstance(processor.Processor(rules),
                       processor.Processor))
    
    def test_version(self):
        # Should be same as bibcat
        self.assertEqual(processor.__version__,
                         bibcat.__version__)

        

    def tearDown(self):
        pass 

class TestRDFMappingLanguageProcessorGenerateTerms(unittest.TestCase):

    def setUp(self):
        self.processor = processor.Processor(
            rml_rules=os.path.join(FIXURES_PATH,
                                   "rml-basic.ttl"))
        self.rr = rdflib.Namespace("http://www.w3.org/ns/r2rml#")
        self.test_map = SimpleNamespace()
        self.test_map.reference = None


    def test_default(self):
        self.assertRaises(KeyError, self.processor.generate_term)

    def test_termType_bnode(self):
        self.test_map.termType = self.rr.BlankNode
        self.assertIsInstance(
            self.processor.generate_term(term_map=self.test_map), 
            rdflib.BNode)

    def test_default_datatype(self):
        self.test_map.template = "{test_default}"
        term = self.processor.generate_term(term_map=self.test_map, test_default=1)
        self.assertIsInstance(term, rdflib.URIRef)

    def test_xsd_date_datatype(self):
        self.test_map.datatype = rdflib.XSD.date
        self.test_map.template = "{test_literal}"
        term = self.processor.generate_term(term_map=self.test_map, test_literal="2017")
        self.assertEqual(term.datatype,
            rdflib.XSD.date)

    def test_xsd_datetime_datatype(self):
        self.test_map.datatype = rdflib.XSD.dateTime
        self.test_map.template = "{test_literal}"
        term = self.processor.generate_term(term_map=self.test_map, test_literal="2017-06-19")
        self.assertEqual(term.datatype,
            rdflib.XSD.dateTime)

    def test_xsd_string_datatype(self):
        self.test_map.datatype = rdflib.XSD.string
        self.test_map.template = "{test_literal}"
        term = self.processor.generate_term(
            term_map=self.test_map, 
            test_literal="A fine string")
        self.assertEqual(term.datatype,
            rdflib.XSD.string)

    def test_template_default_literal(self):
        literal_value = "Default Literal String"
        self.test_map.template = "{test_literal}"
        term = self.processor.generate_term(
            term_map=self.test_map,
            test_literal=literal_value)
        self.assertEqual(str(term),
            literal_value)

    def test_template_default_url(self):
        self.test_map.template = "http://test.io/{id}"
        uri_term = self.processor.generate_term(
            term_map=self.test_map,
            id="1234")
        self.assertEqual(str(uri_term),
            "http://test.io/1234")

      
    def test_template_errors(self):
        self.test_map.template = "{test_literal}"
        self.assertRaises(KeyError, 
            self.processor.generate_term,
            term_map=self.test_map,
            test_not_literal="1234")

    def tearDown(self):
        pass


class TestGetObjectInternalFunction(unittest.TestCase):

    def setUp(self):
        pass

    def test_default_error(self):
        # Binding Required
        self.assertRaises(TypeError, processor.__get_object__) 

    def test_None(self):
        self.assertEqual(processor.__get_object__(None), None)

    def test_simple_binding(self):
        binding = {"value": "1234"}
        self.assertEqual(processor.__get_object__(binding),
            rdflib.Literal(binding.get('value')))

    def tearDown(self):
        pass

class Test__graph__Method(unittest.TestCase):

    def setUp(self):
        self.processor = processor.Processor(
            rml_rules=os.path.join(FIXURES_PATH,
                                   "rml-basic.ttl"))

    def test_default(self):
        self.assertIsInstance(self.processor.__graph__(),
                              rdflib.Graph)

    def test_namespaces(self):
        new_graph = self.processor.__graph__()
        self.assertListEqual(
            [n for n in new_graph.namespace_manager.namespaces()], 
            [n for n in self.processor.rml.namespace_manager.namespaces()])

    def tearDown(self):
        pass

class Test__handle_parents__Method(unittest.TestCase):

    def setUp(self):
        self.processor = processor.Processor(
            rml_rules=os.path.join(FIXURES_PATH,
                                   "rml-basic.ttl"))

    def test_errors(self):
        self.assertRaises(KeyError, self.processor.__handle_parents__)
        self.assertRaises(KeyError, 
            self.processor.__handle_parents__,
            parent_map = SimpleNamespace())

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main() 



"""BIBFRAME 2.0 Work Generator


"""
import datetime
import rdflib
import requests
try:
    from .generator import Generator, new_graph, NS_MGR
    from .sparql import DELETE_WORK_BNODE 
    from .sparql import FILTER_WORK_CREATOR, FILTER_WORK_TITLE 
    from .sparql import GET_AVAILABLE_INSTANCES, GET_INSTANCE_CREATOR 
    from .sparql import GET_INSTANCE_TITLE, GET_INSTANCE_WORK_BNODE_PROPS
    from .sparql import DELETE_COLLECTION_BNODE, FILTER_COLLECTION
    from .sparql import GET_AVAILABLE_COLLECTIONS
except SystemError:
    try:
        from generator import Generator, new_graph, NS_MGR
        from sparql import DELETE_WORK_BNODE 
        from sparql import FILTER_WORK_CREATOR, FILTER_WORK_TITLE  
        from sparql import GET_AVAILABLE_INSTANCES, GET_INSTANCE_CREATOR
        from sparql import GET_INSTANCE_TITLE, GET_INSTANCE_WORK_BNODE_PROPS
        from sparql import DELETE_COLLECTION_BNODE, FILTER_COLLECTION 
        from sparql import GET_AVAILABLE_COLLECTIONS
    except ImportError:
        pass


__author__ = "Jeremy Nelson, Mike Stabile"


class WorkError(Exception):

    def __init__(self, reason):
        self.reason = reason

    def __str__(self):
        return str(self.reason)

class CollectionGenerator(Generator):
    """Queries triplestore for BIBFRAME 2.0 Instances that are related to
    collections that are blank nodes. Generator reifies Collection as a bf:Work
    with the rdfs:label triple and triples for each bf:Instance."""

    def __init__(self, **kwargs):
        super(CollectionGenerator, self).__init__(**kwargs)

    def __generate_collection__(self, **kwargs):
        
        """Generates new Collection for the Instance in the triplestore

        Keyword args:
           instance(rdflib.URIRef): URI of Instance
           label(rdflib.Literal): Literal string of RDF label
        """
        instance = kwargs.get('instance')
        label = kwargs.get('label')
        if instance is None or label is None:
            raise WorkError("Generate Collection both instance and label")
        collection_uri = self.__generate_uri__()
        collection_graph = new_graph()
        for type_of in [NS_MGR.pcdm.Collection, NS_MGR.bf.Work]:
            collection_graph.add((collection_uri, 
                NS_MGR.rdf.type,
                type_of))
        collection_graph.add((collection_uri,
            NS_MGR.rdfs.label,
            label))
        collection_graph.add((collection_uri,
            NS_MGR.bf.hasPart,
            instance))
        result = requests.post(
            self.triplestore_url,
            data=collection_graph.serialize(format='turtle'),
            headers={"Content-Type": "text/turtle"})
        if result.status_code > 399:
            raise WorkError("Failed to add {} to triplestore".format(
                collection_uri))
        return collection_uri



    def __handle_collections__(self, **kwargs):
        """Queries triplestore for collections to match
        existing organizations with collections matching a label or
        other identification.
        
        Keyword args:
            instance(rdflib.URIRef): URI of Instance
            organization(rdflib.URIRef): URI of Organization
            item(rdflib.URIRef): URI of Item
            rdfs_label(rdflib.Literal): Literal string of RDFS label

        Returns:
            list: List of Collection URIs
        """
        instance = kwargs.get('instance')
        org = kwargs.get('organization')
        item = kwargs.get('item')
        label = kwargs.get('rdfs_label')
        query = FILTER_COLLECTION.format(item, org, label)
        result = requests.post(self.triplestore_url,
                data={"query": query,
                      "format": "json"})
        if result.status_code > 399:
            raise WorkError("Failed to run {}".format(query))
        bindings = result.json().get('results').get('bindings')
        if len(bindings) < 1:
            return [self.__generate_collection__(
                instance=instance,
                label=label),]
        else:
            collections = []
            for row in bindings:
                collection_uri = rdflib.URIRef(
                    row.get('collection').get('value'))
                existing_instance = rdflib.URIRef(
                    row.get('instance').get('value'))
                if existing_instance == instance:
                    continue
                collections.append(collection_uri)
                update_graph = new_graph()
                new_graph.add((collection_uri,
                    NS_MGR.bf.hasPart,
                    instance))
                result = requsts.post(
                    self.triplestore_url,
                    data=new_graph.serialize(format='turtle'),
                    headers={"Content-Type": "text/turtle"})
            return collections

    def run(self):
        """Runs Collection Generator"""
        result = requests.post(self.triplestore_url,
                data={"query": GET_AVAILABLE_COLLECTIONS,
                      "format": "json"})
        if result.status_code > 399:
            raise WorkError("Failed to run sparql query")

        bindings = result.json().get('results').get('bindings')
        for row in bindings:
            instance_uri = rdflib.URIRef(row.get('instance').get('value'))
            org_uri = rdflib.URIRef(row.get('org').get('value'))
            item_uri = rdflib.URIRef(row.get('item').get('value'))
            label = rdflib.Literal(row.get('label').get('value'))
            #! Should check for language in label
            collections = self.__handle_collections__(
                instance=instance_uri, 
                item=item_uri,
                organization=org_uri, 
                rdfs_label=label)
            # Now remove existing BNode's properties from the BF Instance
            delete_result = requests.post(
                self.triplestore_url,
                data=DELETE_COLLECTION_BNODE.format(instance_uri),
                headers={"Content-Type": "application/sparql-update"})
            if delete_result.status_code > 399:
                raise WorkError("Cannot Delete Collection blank nodes for {}\n{}".format(
                    instance_uri, delete_result.text))





class WorkGenerator(Generator):
    """Queries BIBFRAME 2.0 triplestore for Instances with missing Works or
    works that are blank nodes, and first attempts to resolve the Instance to
    an existing Work and ifGET_INSTANCE_WORK_BNODE_PROPS that fails, creates a new BIBFRAME Work based the
    Instance's information."""


    def __init__(self, **kwargs):
        """

        Keyword args:
            url (str):  URL for the triplestore, defaults to localhost
                        Blazegraph instance
        """
        self.rules = rdflib.Graph()
        self.matched_works = []
        self.processed = {}
        self.creator_codes = kwargs.get(
            'creator_codes', 
            ['aus', 'aut', 'cre'])
        super(WorkGenerator, self).__init__(**kwargs)

    def __add_creators__(self, work_graph, work_uri, instance_uri):
        """Method takes a new work graph and instance uri, queries for
        relators:creators of instance uri and adds values to work graph
        
         Args:
            work_graph(rdflib.Graph): RDF Graph of new BF Work
            instance_uri(rdflib.URIRef): URI of BF Instance
        """
        instance_key = str(instance_uri)
        if instance_key in self.processed: 
            for code in self.creator_codes:
                if not code in self.processed[instance_key]:
                    continue
                relator = getattr(NS_MGR.relators, code)
                for agent_uri in self.processed[instance_key][code]:
                    work_graph.add((work_uri, 
                                    relator, 
                                    agent_uri))


    def __add_work_title__(self, work_graph, work_uri, instance_uri):
        """Method takes a new work graph and instance uri, queries for
        bf:InstanceTitle of instance uri and adds values to work graph

        Args:
            work_graph(rdflib.Graph): RDF Graph of new BF Work
            instance_uri(rdflib.URIRef): URI of BF Instance
        """
        instance_key = str(instance_uri)
        if instance_key in self.processed and\
        "title" in self.processed[instance_key]:
            work_title_bnode = rdflib.BNode()
            work_graph.add((work_uri,  NS_MGR.bf.title, work_title_bnode))
            work_graph.add((work_title_bnode, 
                            NS_MGR.rdf.type, 
                            NS_MGR.bf.WorkTitle))
            for row in self.processed[instance_key]["title"]:
                main_title, subtitle = row["mainTitle"], row["subtitle"]
                work_graph.add((work_title_bnode,
                                NS_MGR.bf.mainTitle,
                                rdflib.Literal(main_title)))
                if subtitle:
                    work_graph.add((work_title_bnode,
                                    NS_MGR.bf.subtitle,
                                    rdflib.Literal(subtitle)))
                
       

    def __copy_instance_to_work__(self, instance_uri, work_uri):
        """Method takes an instance_uri and work_uri, copies all of the 
        properties in any bf:Work blank nodes in the instance to the work_uri,
        and then deletes the bf:Work blank node and properties from the 
        instance_uri.

        Args:
            instance_uri(rdflib.URIRef): URI of Instance
            work_uri(rdflib.URIRef): URI of Work
        """
        if instance_uri == work_uri:
            raise ValueError(
                "Instance and Work URIs cannot match uri={}".format(
                    instance_uri))
        work_properties_result = requests.post(
            self.triplestore_url,
            data={"query": GET_INSTANCE_WORK_BNODE_PROPS.format(instance_uri),
                  "format": "json"})
        work_properties_bindings = work_properties_result.json()\
             .get("results").get("bindings")
        work_graph = new_graph()
        work_graph.add((instance_uri, NS_MGR.bf.instanceOf, work_uri))
        for row in work_properties_bindings:
            if 'pred' not in row or 'obj' not in row:
                continue
            predicate = rdflib.URIRef(row.get("pred").get("value"))
            obj_type = row.get("obj").get("type")
            obj_raw_val = row.get("obj").get("value")
            if obj_type.startswith("literal"):
                if predicate == rdflib.RDF.type:
                    # Skip all literals 
                    continue
                obj_ = rdflib.Literal(obj_raw_val)
            else:
                obj_ = rdflib.URIRef(obj_raw_val)
            work_graph.add((work_uri, predicate, obj_))
        self.__add_work_title__(work_graph, work_uri, instance_uri)
        self.__add_creators__(work_graph, work_uri, instance_uri)
        update_result = requests.post(
            self.triplestore_url,
            data=work_graph.serialize(format='turtle'),
            headers={"Content-Type": "text/turtle"})
        # Now remove existing BNode's properties from the BF Instance
        delete_result = requests.post(
            self.triplestore_url,
            data=DELETE_WORK_BNODE.format(instance_uri),
            headers={"Content-Type": "application/sparql-update"})
        if delete_result.status_code > 399:
            raise ValueError("Cannot Delete Work blank nodes for {}\n{}".format(
                instance_uri, delete_result.text))
      


    def __generate_work__(self, instance_uri):
        """Internal method takes an BIBFRAME Instance URI, queries triplestore
        for matches based on the loaded rules.

        Args:
            instance_uri(str): URI of BIBFRAME Instance
        Returns:
            rdflib.URIRef: New or existing Work URI
        """
        work_uri = None
        self.matched_works = []
        self.__similar_titles__(instance_uri)
        self.__similar_creators__(instance_uri)
        candidate_works = list(set(self.matched_works))
        if len(candidate_works) < 1:
            work_uri = self.__generate_uri__()
            work_graph = new_graph()
            work_graph.add((work_uri, NS_MGR.rdf.type, NS_MGR.bf.Work))
        else:
            # Takes the top match
            work_uri = rdflib.URIRef(candidate_works[0])
        return work_uri


    def __similar_creators__(self, uri): 
        """Takes an BF Instance URI, extracts creator info and then
        queries triplestore for similar works with the same creator

        Args:
            uri(str): URI of BIBFRAME Instance

        """
        for code in self.creator_codes:
            sparql = GET_INSTANCE_CREATOR.format(
                        uri,
                        getattr(NS_MGR.relators, code))
            instance_creator_result = requests.post(
                self.triplestore_url,
                data={"query": sparql,
                      "format": "json"})
            instance_creator_bindings = instance_creator_result.json()\
                .get("results").get("bindings")
            for row in instance_creator_bindings:
                creator_name = row.get("name", {}).get("value")
                creator_url = row.get("creator", {}).get("value")
                if creator_name is None or creator_url is None:
                    continue
                creator_uri = rdflib.URIRef(creator_url)
                instance_key = str(uri)
                if instance_key in self.processed:
                    if code in self.processed[instance_key]:
                        self.processed[instance_key][code].append(
                            creator_uri)
                    else:
                        self.processed[instance_key][code] = [
                            creator_uri,]
                else:
                    self.processed[instance_key] = {code: [
                            creator_uri,]}
                     
                work_creator_result = requests.post(
                    self.triplestore_url,
                    data={"query": FILTER_WORK_CREATOR.format(creator_name),
                          "format": "json"})
                work_creator_bindings = work_creator_result.json()\
                .get("results").get("bindings")
                if len(work_creator_bindings) < 1:
                    continue
                for work_row in work_creator_bindings:
                    self.matched_works.append(
                        work_row.get("work").get("value"))



    def __similar_titles__(self, uri):
        """Takes an BF Instance URI, extracts titles info and then
        queries triplestore for similar works with the same title

        Args:
            uri(str): URI of BIBFRAME Instance
        """
        instance_title_result = requests.post(
            self.triplestore_url,
            data={"query": GET_INSTANCE_TITLE.format(uri),
                  "format": "json"})
        instance_title_bindings = instance_title_result.json()\
            .get("results").get("bindings")
        for row in instance_title_bindings:
            main_title, subtitle = row.get("mainTitle", {}).get("value"), None
            if main_title is None:
                continue
            if "subtitle" in row:
                subtitle = row.get("subtitle").get("value")
            if uri in self.processed and "title" in self.processed[uri]:
                self.processed[uri]["title"].append(
                    {"mainTitle": main_title,
                     "subtitle": subtitle})
            else:
                self.processed[uri] = {"title": [{"mainTitle": main_title,
                                                  "subtitle": subtitle}]}
            escaped_title = main_title.replace('"', '\"')
            query = FILTER_WORK_TITLE.format(escaped_title)
            work_title_result = requests.post(
                self.triplestore_url,
                #! Need to add subtitle to SPARQL query
                data={"query": query,
                      "format": "json"})
            if work_title_result.status_code > 399:
                print("Error with similar title {}".format(main_title))
            work_title_bindings = work_title_result.json()\
               .get("results").get("bindings")
            if len(work_title_bindings) < 1:
                continue
            for work_row in work_title_bindings:
                self.matched_works.append(work_row.get("work").get("value"))


    def harvest_instances(self):
        """
        Harvests all BIBFRAME Instances that have an Blank Node for the
        isInstanceOf property.
        
        #! for performance considerations may need to run sparql query that
        limits the batch size
        """
        result = requests.post(
            self.triplestore_url,
            data={"query": GET_AVAILABLE_INSTANCES,
                  "format": "json"})
        if result.status_code > 399:
            raise WorkError("WorkGenerator failed to query {}".format(
                self.triplestore_url))
        bindings = result.json().get('results').get('bindings')
        start = datetime.datetime.utcnow()
        print("Started Processing at {} for {} Instances".format(
            start,
            len(bindings)))
        for i, row in enumerate(bindings):
            instance_url =  row.get('instance').get('value')
            work_uri = self.__generate_work__(instance_url)
            self.__copy_instance_to_work__(
                rdflib.URIRef(instance_url), 
                work_uri)
            if not i%10 and i > 0:
                print(".", end="")
            if not i%100:
                print(i, end="")
        end = datetime.datetime.utcnow()
        print("Finished Processing at {}, total time={} mins".format(
            end,
            (end-start).seconds / 60.0))
            


    def run(self):
        """Runs work generator on triplestore"""

        self.harvest_instances()





import datetime


import logging
import os
import pymarc
import rdflib
import sys
import unittest
import uuid

sys.path.append(os.path.abspath(os.path.curdir))
import ingesters
import ingesters.marc as marc

marc.MLOG_LVL = logging.CRITICAL
logging.getLogger("requests").setLevel(logging.CRITICAL)
logging.getLogger("urllib3").setLevel(logging.CRITICAL)

class TestMARC__handle_linked_pattern__(unittest.TestCase):

    def setUp(self):
        self.ingester = marc.MARCIngester(record=pymarc.Record())

    def test_exists(self):
        self.assertTrue(hasattr(self.ingester, "__handle_linked_pattern__"))

    def test_no_keywords(self):
        self.assertIsNone(
            self.ingester.__handle_linked_pattern__())

    def tearDown(self):
        self.ingester.graph.close()


class TestDeduplicatingInstances(unittest.TestCase):

    def setUp(self):
        self.ingester = marc.MARCIngester(record=pymarc.Record())


    def test_default_deduplicate_instances(self):
        self.ingester.deduplicate_instances()


class TestDeduplicatingAgents(unittest.TestCase):

    def setUp(self):
        self.ingester = marc.MARCIngester(record=pymarc.Record())

    def test_default_deduplicate_agents(self):
        self.ingester.deduplicate_agents(
            ingesters.ingester.NS_MGR.schema.oclc, 
            ingesters.ingester.NS_MGR.bf.Agent)


class TestMatchMARC(unittest.TestCase):

    def setUp(self):
        self.marc_record = pymarc.Record()
        self.marc_record.add_field(
            pymarc.Field('245', 
                ['1', '0'],
                ['a', 'This is a test:',
                 'b', 'and subtitle']))
        self.ingester = marc.MARCIngester(record=self.marc_record)


    def test_match_245_mainTitle(self):
        self.assertEqual(
            self.ingester.match_marc('M24510a'),
            ["This is a test:",])

    def test_match_245_subtitle(self):
        self.assertEqual(
            self.ingester.match_marc('M24510b'),
            ["and subtitle"])

class TestMARCUpdateLinkedClasses(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test.org/entity/1")
        self.ingester = marc.MARCIngester(record=pymarc.Record())

    def test_default_method(self):
        self.ingester.update_linked_classes(
            ingesters.ingester.NS_MGR.bf.Item,
            self.entity)

    def tearDown(self):
        pass

class TestMARCUpdateDirectProperties(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test.org/entity/1")
        self.ingester = marc.MARCIngester(record=pymarc.Record())


    def test_default_method(self):
        self.ingester.update_direct_properties(
            ingesters.ingester.NS_MGR.bf.Instance,
            self.entity)

class TestMARCUpdateOrderedLinkedClasses(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test.org/entity/1")
        self.ingester = marc.MARCIngester(record=pymarc.Record())
       
    def test_default_method(self):
        self.ingester.update_ordered_linked_classes(
            ingesters.ingester.NS_MGR.bf.Item,
            self.entity)



if __name__ == '__main__':
    unittest.main()
    




import logging
import os
import rdflib
import requests
import sys
import unittest
import xml.etree.ElementTree as etree

sys.path.append(os.path.abspath(os.path.curdir))
BIBCAT_BASE = os.path.abspath(
    os.path.split(
        os.path.dirname(__file__))[0])
PROJECT_BASE = os.path.split(BIBCAT_BASE)[0]
sys.path.append(PROJECT_BASE)
try:
    from instance import config
except ModuleNotFoundError:
    class Config(object):
        def __init__(self):
            self.BASE_URL = "http://bibcat.org/"
            self.TRIPLESTORE_URL = "http://localhost:9999/blazegraph/sparql"
    config = Config()
import ingesters
import ingesters.mods as mods
from ingesters.ingester import Ingester, NS_MGR

ingesters.MLOG_LVL = logging.CRITICAL
logging.getLogger("requests").setLevel(logging.CRITICAL)
logging.getLogger("urllib3").setLevel(logging.CRITICAL)

SAMPLE_MODS = etree.XML("""<mods xmlns="http://www.loc.gov/mods/v3" 
 xmlns:mods="http://www.loc.gov/mods/v3" 
 xmlns:xlink="http://www.w3.org/1999/xlink" 
 xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <titleInfo>
    <title>Arthur J. Kew and Ida Rosalie Fursman</title>
  </titleInfo>
  <typeOfResource>mixed material</typeOfResource>
<name type="personal">
    <namePart>Kew, Arthur J.</namePart>
    <role>
      <roleTerm authority="marcrelator" type="text">creator</roleTerm>
    </role>
  </name>
  <name type="personal">
    <namePart>Fursman, Ida Rosalie</namePart>
    <role>
      <roleTerm authority="marcrelator" type="text">creator</roleTerm>
    </role>
  </name>
  <genre authority="marcgt">letter</genre>
  <genre authority="marcgt">picture</genre>
  <location><url>http://example.edu/1234</url></location>
</mods>""")

SAMPLE_MODS_THESIS = etree.XML("""<mods xmlns="http://www.loc.gov/mods/v3" 
 xmlns:mods="http://www.loc.gov/mods/v3" 
 xmlns:xlink="http://www.w3.org/1999/xlink" 
 xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
    <titleInfo>
        <title>Sirena Selena, Queen of Dreams: A Responsible Translation of Difference and Unspoken Rhetorics</title>
    </titleInfo>
    <genre authority="marcgt">thesis</genre>
    <name type="corporate">
        <namePart>Colorado College</namePart>
        <role>
            <roleTerm type="text" authority="marcrt">degree grantor</roleTerm>
        </role>
    </name>
</mods>""")

CREATOR_XPATH = "mods:name[@type='personal']/mods:role[mods:roleTerm='creator']/../mods:namePart"
SELECT_GENRES = NS_MGR.prefix() + """
SELECT ?genre
WHERE {
    ?subj rdf:type bf:GenreForm .
    ?subj rdf:value ?genre .
}"""

SELECT_ORG_LABEL = NS_MGR.prefix() + """
SELECT ?label 
WHERE {
    ?subj rdf:type bf:Organization .
    ?subj rdfs:label ?label .
}"""

class TestMODS__handle_linked_pattern__(unittest.TestCase):

    def setUp(self):
        self.ingester = mods.MODSIngester(source=SAMPLE_MODS)
        self.entity = self.ingester.__generate_uri__()

    def test_exists(self):
        self.assertTrue(hasattr(self.ingester, "__handle_linked_pattern__"))

    def test_no_keywords(self):
        self.assertIsNone(self.ingester.__handle_linked_pattern__())


    def test_creator_pattern(self):
        self.ingester.__handle_linked_pattern__(
            entity=self.entity,
            rule=rdflib.Literal(CREATOR_XPATH),
            destination_class=NS_MGR.bf.Person,
            destination_property=NS_MGR.schema.alternativeName,
            target_class=NS_MGR.bf.Instance,
            target_property=NS_MGR.relators.cre)
        self.assertEqual(len(self.ingester.graph), 4)

    def test_genre_pattern(self):
        self.ingester.__handle_linked_pattern__(
            entity=self.entity,
            rule=rdflib.Literal("mods:genre[@authority='marcgt']"),
            destination_class=NS_MGR.bf.GenreForm,
            destination_property=NS_MGR.rdf.value,
            target_class=NS_MGR.bf.Instance,
            target_property=NS_MGR.bf.genreForm)
        genres = []
        for row in self.ingester.graph.query(SELECT_GENRES):
            genres.append(str(row[0]))
        self.assertListEqual(sorted(genres),
            ["letter", "picture"])
                 

    def test_title_pattern(self):
        self.ingester.__handle_linked_pattern__(
            entity=self.entity,
            rule=rdflib.Literal("mods:titleInfo/mods:title"),
            destination_class=NS_MGR.bf.InstanceTitle,
            destination_property=NS_MGR.bf.mainTitle,
            target_class=NS_MGR.bf.Instance,
            target_property=NS_MGR.bf.title)
        title = None
        for row in self.ingester.graph.objects(predicate=NS_MGR.bf.mainTitle):
            title = row
        self.assertEqual(
            str(title),
            "Arthur J. Kew and Ida Rosalie Fursman")

    def tearDown(self):
        self.ingester.graph.close()


class Test__handle_linked_bnode__(unittest.TestCase):

    def setUp(self):
        self.ingester = mods.MODSIngester(source=SAMPLE_MODS_THESIS)
        self.entity = self.ingester.__generate_uri__()

    def test_exists(self):
        self.assertTrue(hasattr(self.ingester, "__handle_linked_bnode__"))

    def test_degreeGrantor(self):
        degree_grantor = getattr(NS_MGR.bc, "mods-degreeGrantor")
        degree_grantor_bnode = self.ingester.rules_graph.value(
            subject=degree_grantor,
            predicate=NS_MGR.kds.destPropUri)
        self.ingester.__handle_linked_bnode__(
            entity=self.entity,
            bnode=degree_grantor_bnode,
            destination_class=NS_MGR.bf.Dissertation,
            target_property=NS_MGR.bf.dissertation,
            target_subject=degree_grantor)
        org_labels = []
        for row in self.ingester.graph.query(SELECT_ORG_LABEL):
            org_labels.append(str(row[0]))
        self.assertEqual(
            org_labels[0],
            "Colorado College")
            
    def test_no_keywords(self):
        pass

       
    def tearDown(self):
        self.ingester.graph.close()
       

class Test__handle_pattern__(unittest.TestCase):

    def setUp(self):
        self.ingester = mods.MODSIngester(source=SAMPLE_MODS)
        self.entity = self.ingester.__generate_uri__()
        self.cc = rdflib.URIRef("http://coloradocollege.edu/")
        bc_org = getattr(NS_MGR.kds, "bf-Organization")
        self.ingester.rules_graph.add((
            bc_org,
            NS_MGR.rdf.type,
            NS_MGR.kds.PropertyLinker))
        self.held_by = rdflib.BNode()
        self.ingester.rules_graph.add((
            bc_org,
            NS_MGR.kds.destPropUri,
            self.held_by))
        self.ingester.rules_graph.add((
            self.held_by,
            NS_MGR.bf.heldBy,
            self.cc))
        self.ingester.rules_graph.add((
            bc_org,
            NS_MGR.kds.destClassUri,
            NS_MGR.bf.Item)) 
            

    def test_exists(self):
        self.assertTrue(
            hasattr(self.ingester,
                    '__handle_pattern__'))

    def test_location_url(self):
        self.ingester.__handle_pattern__(
            self.entity,
            rdflib.Literal("mods:location/mods:url"),
            NS_MGR.schema.url)
        self.assertEqual(
            str(self.ingester.graph.value(subject=self.entity,
                predicate=NS_MGR.schema.url)),
            "http://example.edu/1234")

    def test_add_org_item(self):
        self.ingester.graph.add((
            self.entity,
            NS_MGR.rdf.type,
            NS_MGR.bf.Item))
        self.ingester.__handle_pattern__(
            self.entity,
            None,
            self.held_by)
        self.assertEqual(
            self.ingester.graph.value(subject=self.entity,
                predicate=NS_MGR.bf.heldBy),
            self.cc)

    def test_no_args(self):
        self.assertRaises(
            TypeError,
            self.ingester.__handle_pattern__)
        

    def tearDown(self):
        self.ingester.rules_graph.close()

class TestInitMODSIngester(unittest.TestCase):

    def setUp(self):
        self.ingester = mods.MODSIngester()

    def test_defaults(self):
        if hasattr(config, "BASE_URL"):
            self.assertEqual(
                self.ingester.base_url,
                config.BASE_URL)
        self.assertEqual(
            len(self.ingester.graph),
            0)
        self.assertTrue(
            len(self.ingester.rules_graph) > 1)
        self.assertIsNone(self.ingester.source)
        self.assertIn(
            self.ingester.triplestore_url,
            ['http://localhost:9999/blazegraph/sparql',
             config.TRIPLESTORE_URL])

    def tearDown(self):
        self.ingester.graph.close()


class TestMODSUpdateDirectProperties(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test.org/entity/1")
        self.ingester = mods.MODSIngester()


    def test_default_method(self):
        self.ingester.update_direct_properties(
            NS_MGR.bf.Instance,
            self.entity)

    def tearDown(self):
        self.ingester.graph.close()


class TestMODSUpdateLinkedClasses(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test.org/entity/1")
        self.ingester = mods.MODSIngester()

    def test_default_method(self):
        self.ingester.update_linked_classes(
            NS_MGR.bf.Item,
            self.entity)

    def tearDown(self):
        self.ingester.graph.close()


class TestMODSUpdateOrderedLinkedClasses(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test.org/entity/1")
        self.ingester = mods.MODSIngester()
       
    def test_default_method(self):
        self.ingester.update_ordered_linked_classes(
            NS_MGR.bf.Item,
            self.entity)

    def tearDown(self):
        self.ingester.graph.close()


class TestDeduplicateAgents(unittest.TestCase):

    def setUp(self):
        self.ingester = mods.MODSIngester(source=SAMPLE_MODS)
        self.entity =  self.ingester.__generate_uri__()

    def test_defaults_no_transform(self):
        self.assertEqual(len(self.ingester.graph), 0)
        self.ingester.deduplicate_agents(None, None)
        self.assertEqual(len(self.ingester.graph), 0)

    #! NEED to mock triplestore 
    def test_defaults_transform(self):
        self.ingester.transform()
        

    def tearDown(self):
        self.ingester.rules_graph.close()

if __name__ == '__main__':
    unittest.main()



"""Tests oai_pmh module's OAI_PMH base class and Islandora subclass"""
__author__ = "Jeremy Nelson"

import os
import sys
import unittest
from unittest import mock
try:
    from bibcat.ingesters.oai_pmh import OAIPMHIngester, IslandoraIngester
except ImportError:
    BIBCAT_BASE = os.path.abspath(".")
    sys.path.append(BIBCAT_BASE)
    from bibcat.ingesters.oai_pmh import OAIPMHIngester, IslandoraIngester

# Mock of OAI-PMH Fee
def mocked_oai_pmh(*args, **kwargs):
    class MockOAIPMHResponse(object):

        def __init__(self, xml, status_code):
            self.xml = xml
            self.status_code = status_code

        @property
        def text(self):
            return self.xml  

        
    fixures_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), 
        "fixures")
    if args[0] == 'http://bibcat.org/oai2?verb=ListMetadataFormats':
        with open(os.path.join(fixures_path, 
                               "list-metadata-formats.xml")) as fo:
            list_metadata_xml = fo.read()
        return MockOAIPMHResponse(list_metadata_xml, 200)
    elif args[0] == 'http://bibcat.org/oai2?verb=ListIdentifiers&metadataPrefix=oai_dc':
        with open(os.path.join(fixures_path, "first-dc-ids.xml")) as fo:
            first_dc_ids_xml = fo.read()
        return MockOAIPMHResponse(first_dc_ids_xml, 200)
    elif args[0] == 'http://bibcat.org/oai2?verb=ListIdentifiers&resumptionToken=2340121':
        with open(os.path.join(fixures_path, "final-dc-ids.xml")) as fo:
            final_dc_ids = fo.read()
        return MockOAIPMHResponse(final_dc_ids, 200)
    else:
        return MockOAIPMHResponse(None, 404)


class TestOAI_PMHIngester(unittest.TestCase):

    def setUp(self):
        """Creates test environment"""
        pass

    def test_defaults(self):
        self.assertRaises(ValueError, OAIPMHIngester)

    def test_class_constants(self):
        self.assertTrue(hasattr(OAIPMHIngester, "IDENT_XPATH"))
        self.assertTrue(hasattr(OAIPMHIngester, "TOKEN_XPATH")) 

    @mock.patch("bibcat.ingesters.oai_pmh.requests.get", 
        side_effect=mocked_oai_pmh)
    def test_init_repo(self, mock_get):
        oai_ingester = OAIPMHIngester(repository='http://bibcat.org/oai2')
        self.assertEqual(oai_ingester.oai_pmh_url,
                         'http://bibcat.org/oai2')
        self.assertEqual(oai_ingester.metadataPrefix, "oai_dc")


    def tearDown(self):
        pass

class TestIslandoraIngester(unittest.TestCase):

    def setUp(self):
        pass

    def test_defaults(self):
        self.assertRaises(KeyError, IslandoraIngester)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



__author__ = "Jeremy Nelson"


import unittest
import rdflib
from bibcat.rml.processor import XMLProcessor

class TestXMLProcessorInit(unittest.TestCase):

    def setUp(self):
        pass

    def test_default_execptions(self):
        self.assertRaises(Exception, XMLProcessor)
        

    def test_default_base(self):
        base_rules = rml_rules=["bibcat-base.ttl"]
        processor = XMLProcessor(rml_rules=base_rules)
        self.assertEqual(len(processor.rml), 16)

   

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main() 



"""Python RDF Mapping Language Processor

>>> import bibcat.rml.processor as processor

"""

__author__ = "Jeremy Nelson"

# Standard Python Modules
import collections
import datetime
import json
import os
from types import SimpleNamespace

# 3rd party modules
import bibcat
import rdflib
import requests

import jsonpath_ng
from bibcat.maps import get_map

BIBCAT_BASE = os.path.abspath(
    os.path.split(
        os.path.dirname(__file__))[0])
NS_MGR = SimpleNamespace()
PREFIX = None
__version__ = bibcat.__version__

try:
    from lxml import etree
except ImportError:
    import xml.etree.ElementTree as etree

class Processor(object):
    """Base class for RDF Mapping Language Processors, child classes
    encapsulate different types of Data sources


    """

    def __init__(self, rml_rules):
        self.rml = rdflib.Graph()
        if isinstance(rml_rules, list):
            for rule in rml_rules:
                # First check if rule exists on the filesystem
                if os.path.exists(rule):
                    with open(rule) as file_obj:
                        raw_rule = file_obj.read()
                else:
                    raw_rule = get_map(rule).decode()
                self.rml.parse(data=raw_rule,
                               format='turtle')
        elif isinstance(rml_rules, (rdflib.Graph, rdflib.ConjunctiveGraph)):
            self.rml = rml_rules
        elif os.path.exists(rml_rules):
            self.rml.parse(rml_rules, format='turtle')
        else:
            self.rml.parse(data=get_map(rml_rules).decode(), format='turtle')
        # Populate Namespaces Manager
        for prefix, namespace in self.rml.namespaces():
            setattr(NS_MGR, prefix, rdflib.Namespace(namespace))
        self.output, self.source, self.triplestore_url = None, None, None
        self.parents = set()
        self.constants = dict(version=__version__)
        self.triple_maps = dict()
        for row in self.rml.query(GET_TRIPLE_MAPS):
            triple_map_iri = row[0]
            map_key = str(triple_map_iri)
            self.triple_maps[map_key] = SimpleNamespace()
            self.triple_maps[map_key].logicalSource = \
                self.__logical_source__(triple_map_iri)
            self.triple_maps[map_key].subjectMap = \
                self.__subject_map__(triple_map_iri)
            self.triple_maps[map_key].predicateObjectMap = \
                self.__predicate_object_map__(triple_map_iri)

    def __graph__(self):
        """Method returns a new graph with all of the namespaces in
        RML graph"""
        #graph = rdflib.Graph(namespace_manager=self.rml.namespace_manager)
        graph = rdflib.Graph()
        for prefix, name in self.rml.namespaces():
            graph.namespace_manager.bind(prefix, name)
        return graph



    def __generate_delimited_objects__(self, **kwargs):
        """Internal methods takes a subject, predicate, element, and a list
        of delimiters that are applied to element's text and a triples
        for each value is created and associated with the subject.

        Keyword Args:

        -------------
            triple_map: SimpleNamespace
            predicate: URIRef
            element: XML Element
            datatype: XSD Datatype, optional
            delimiters: List of delimiters to apply to string
        """
        triple_map = kwargs.get("triple_map")
        subject = kwargs.get('subject')
        # Subject is blank-node, try to retrieve subject IRI
        predicate = kwargs.get('predicate')
        element = kwargs.get('element')
        datatype = kwargs.get('datatype', None)
        delimiters = kwargs.get('delimiters')
        subjects = []
        for delimiter in delimiters:
            values = element.text.split(delimiter)
            for row in values:
                if datatype is not None:
                    obj_ = rdflib.Literal(row.strip(), datatype=datatype)
                else:
                    obj_ = rdflib.Literal(row.strip())
                if isinstance(subject, rdflib.BNode):
                    new_subject = rdflib.BNode()
                    class_ = triple_map.subjectMap.class_
                    self.output.add((new_subject, NS_MGR.rdf.type, class_))
                    for parent_subject, parent_predicate in self.output.subject_predicates(
                            object=subject):
                        self.output.add((parent_subject, parent_predicate, new_subject))
                else:
                    new_subject = subject
                subjects.append(new_subject)
                self.output.add((new_subject, predicate, obj_))
        return subjects

    def __generate_reference__(self, triple_map, **kwargs):
        """Placeholder method, should be extended by child classes

        Args:

        -----
            triple_map: SimpleNamespace

        Keyword Args:

        -------------
        """
        pass

    def __generate_object_term__(self, datatype, value):
        """Internal method takes a datatype (can be None) and returns
        the RDF Object Term

        Args:

        -----
            datatype: None, or rdflib.URIRef
            value: Varys depending on ingester
        """
        if datatype == NS_MGR.xsd.anyURI:
            term = rdflib.URIRef(value)
        elif datatype:
            term = rdflib.Literal(value, datatype=datatype)
        else:
            term = rdflib.Literal(value)
        return term

    def __handle_parents__(self, **kwargs):
        """Internal method handles parentTriplesMaps

        Keyword args:

        -------------

            parent_map: SimpleNamespace of ParentTriplesMap
            subject: rdflib.URIRef or rdflib.BNode
            predicate: rdflib.URIRef
        """
        parent_map = kwargs.pop("parent_map")
        subject = kwargs.pop('subject')
        predicate = kwargs.pop('predicate')
        parent_objects = self.execute(
            self.triple_maps[str(parent_map)],
            **kwargs)
        for parent_obj in parent_objects:
            self.output.add((
                subject,
                predicate,
                parent_obj))

    def __logical_source__(self, map_iri):
        """Creates a SimpleNamespace for the TripelMap's logicalSource

        Args:

        -----
            map_iri: URIRef
        """
        logical_source = SimpleNamespace()
        logical_src_bnode = self.rml.value(
            subject=map_iri,
            predicate=NS_MGR.rml.logicalSource)
        if logical_src_bnode is None:
            return
        logical_source.source = self.rml.value(
            subject=logical_src_bnode,
	           predicate=NS_MGR.rml.source)
        logical_source.reference_formulations = [r for r in self.rml.objects(
            subject=logical_src_bnode,
            predicate=NS_MGR.rml.referenceFormulation)]
        logical_source.iterator = self.rml.value(
            subject=logical_src_bnode,
            predicate=NS_MGR.rml.iterator)
        query = self.rml.value(
            subject=logical_src_bnode,
            predicate=NS_MGR.rml.query)
        if query is not None:
            logical_source.query = query
        return logical_source

    def __subject_map__(self, map_iri):
        """Creates a SimpleNamespace for the TripleMap's subjectMap and
        populates properties from the RML RDF graph

        Args:

        -----
            map_iri: rdflib.URIRef,TripleMap IRI

        Returns:

        --------
            SimpleNamespace
        """
        subject_map = SimpleNamespace()
        subject_map_bnode = self.rml.value(
            subject=map_iri,
            predicate=NS_MGR.rr.subjectMap)
        if subject_map_bnode is None:
            return
        #! Should look at supporting multple rr:class definitions
        subject_map.class_ = self.rml.value(
            subject=subject_map_bnode,
            predicate=getattr(NS_MGR.rr, "class"))
        subject_map.template = self.rml.value(
            subject=subject_map_bnode,
            predicate=NS_MGR.rr.template)
        subject_map.termType = self.rml.value(
            subject=subject_map_bnode,
            predicate=NS_MGR.rr.termType)
        subject_map.deduplicate = self.rml.value(
            subject=subject_map_bnode,
            predicate=NS_MGR.kds.deduplicate)
        subject_map.reference = self.rml.value(
            subject=subject_map_bnode,
            predicate=NS_MGR.rr.reference)
        return subject_map

    def __predicate_object_map__(self, map_iri):
        """Iterates through rr:predicateObjectMaps for this TripleMap
        creating a SimpleNamespace for each triple map and assigning the
        constant, template, parentTripleMap, reference as properties.

        Args:

        -----
                map_iri:  rdflib.URIRef, TripleMap IRI

        Returns:

        --------
                list:  List of predicate_object Namespace objects
        """
        pred_obj_maps = []
        for pred_obj_map_bnode in self.rml.objects(
                subject=map_iri,
                predicate=NS_MGR.rr.predicateObjectMap):
            pred_obj_map = SimpleNamespace()
            pred_obj_map.predicate = self.rml.value(
                subject=pred_obj_map_bnode,
                predicate=NS_MGR.rr.predicate)
            obj_map_bnode = self.rml.value(
                subject=pred_obj_map_bnode,
                predicate=NS_MGR.rr.objectMap)
            if obj_map_bnode is None:
                continue
            pred_obj_map.constant = self.rml.value(
                subject=obj_map_bnode,
                predicate=NS_MGR.rr.constant)
            pred_obj_map.template = self.rml.value(
                subject=obj_map_bnode,
                predicate=NS_MGR.rr.template)
            pred_obj_map.parentTriplesMap = self.rml.value(
                subject=obj_map_bnode,
                predicate=NS_MGR.rr.parentTriplesMap)
            if pred_obj_map.parentTriplesMap is not None:
                self.parents.add(str(pred_obj_map.parentTriplesMap))
            pred_obj_map.reference = self.rml.value(
                subject=obj_map_bnode,
                predicate=NS_MGR.rr.reference)
            pred_obj_map.datatype = self.rml.value(
                subject=obj_map_bnode,
                predicate=NS_MGR.rr.datatype)
            pred_obj_map.query = self.rml.value(
                subject=obj_map_bnode,
                predicate=NS_MGR.rml.query)
            # BIBCAT Extensions
            pred_obj_map.delimiters = []
            for obj in self.rml.objects(subject=obj_map_bnode,
                                        predicate=NS_MGR.kds.delimiter):
                pred_obj_map.delimiters.append(obj)
            pred_obj_maps.append(pred_obj_map)
        return pred_obj_maps

    def generate_term(self, **kwargs):
        """Method generates a rdflib.Term based on kwargs"""
        term_map = kwargs.pop('term_map')
        if hasattr(term_map, "termType") and\
            term_map.termType == NS_MGR.rr.BlankNode:
            return rdflib.BNode()
        if not hasattr(term_map, 'datatype'):
            term_map.datatype = NS_MGR.xsd.anyURI
        if hasattr(term_map, "template") and term_map.template is not None:
            template_vars = kwargs
            template_vars.update(self.constants)
            # Call any functions to generate values
            for key, value in template_vars.items():
                if hasattr(value, "__call__"):
                    template_vars[key] = value()
            raw_value = term_map.template.format(**template_vars)
            if term_map.datatype == NS_MGR.xsd.anyURI:
                return rdflib.URIRef(raw_value)
            return rdflib.Literal(raw_value,
                                  datatype=term_map.datatype)
        if term_map.reference is not None:
            # Each child will have different mechanisms for referencing the
            # source based
            return self.__generate_reference__(term_map, **kwargs)

    def execute(self, triple_map, **kwargs):
        """Placeholder method should be overridden by child classes"""
        pass

    def run(self, **kwargs):
        """Run method iterates through triple maps and calls the execute
        method"""
        if 'timestamp' not in kwargs:
            kwargs['timestamp'] = datetime.datetime.utcnow().isoformat()
        if 'version' not in kwargs:
            kwargs['version'] = bibcat.__version__
        for map_key, triple_map in self.triple_maps.items():
            if map_key not in self.parents:
                self.execute(triple_map, **kwargs)
        
class CSVProcessor(Processor):
    """CSV RDF Mapping Processor"""

    def __init__(self, **kwargs):
        if "fields" in kwargs:
            self.fields = kwargs.pop("fields")
        if "rml_rules" in kwargs:
            rml_rules = kwargs.pop("rml_rules")
        csv_file = kwargs.pop("csv_file")
        self.reader = csv.DictReader(open(csv_file, 'rb'))
        super(CSVProcessor, self).__init__(rml_rules)

    def __generate_reference__(self, triple_map, **kwargs):
        """Extracts the value of either column by key or by position """
        pass

    def execute(self, triple_map, **kwargs):
        """Method executes mapping between CSV source and
        output RDF

        args:
            triple_map(SimpleNamespace): Triple Map
        """
        pass

    def run(self, **kwargs):
        """Method runs through CSV Reader and applies rules to each
        row.

        """
        
class CSVRowProcessor(Processor):

    def __init__(self, **kwargs):
        if "rml_rules" in kwargs:
            rml_rules = kwargs.pop("rml_rules")
        else:
            rml_rules = []
        super(CSVRowProcessor, self).__init__(rml_rules)

    def __generate_reference__(self, triple_map, **kwargs):
        """Generates a RDF entity based on triple map

        Args:
            triple_map(SimpleNamespace): Triple Map
        """
        raw_value = self.source.get(str(triple_map.reference))
        if raw_value is None or len(raw_value) < 1:
            return
        if hasattr(triple_map, "datatype"):
            if triple_map.datatype == NS_MGR.xsd.anyURI:
                output = rdflib.URIRef(raw_value)
            else:
                output = rdflib.Literal(raw_value,
                    datatype=triple_map.datatype) 
        else:
            output = rdflib.Literal(raw_value)
        return output

    def execute(self, triple_map, **kwargs):
        """Method executes mapping between CSV source and
        output RDF

        args:
            triple_map(SimpleNamespace): Triple Map
        """
        subject = self.generate_term(term_map=triple_map.subjectMap,
                                     **kwargs)
        start_size = len(self.output)
        all_subjects = []
        for pred_obj_map in triple_map.predicateObjectMap:
            predicate = pred_obj_map.predicate
            if pred_obj_map.template is not None:
                object_ = self.generate_term(term_map=pred_obj_map, **kwargs)
                if len(str(object)) > 0:
                    self.output.add((
                        subject,
                        predicate,
                        object_))
             
            if pred_obj_map.parentTriplesMap is not None:
                self.__handle_parents__(
                    parent_map=pred_obj_map.parentTriplesMap,
                    subject=subject,
                    predicate=predicate,
                    **kwargs)
            if pred_obj_map.reference is not None:
                object_ = self.generate_term(term_map=pred_obj_map,
                                             **kwargs)
                if object_ and len(str(object_)) > 0:
                    self.output.add((subject, predicate, object_))
            if pred_obj_map.constant is not None:
                self.output.add((subject, predicate, pred_obj_map.constant))
        finish_size = len(self.output)
        if finish_size > start_size:
            self.output.add((subject, 
                             NS_MGR.rdf.type, 
                             triple_map.subjectMap.class_))
            all_subjects.append(subject)
        return all_subjects



    def run(self, row, **kwargs):
        """Methods takes a row and depending if a dict or list,
        runs RML rules.

        Args:
        -----
            row(Dict, List): Row from CSV Reader
        """
        self.source = row
        self.output = self.__graph__()
        super(CSVRowProcessor, self).run(**kwargs)
        

class JSONProcessor(Processor):
    """JSON RDF Mapping Processor"""

    def __init__(self, **kwargs):
        try:
            rml_rules = kwargs.pop("rml_rules")
        except KeyError:
            rml_rules = []
        super(JSONProcessor, self).__init__(rml_rules)

    def __generate_reference__(self,  triple_map, **kwargs):
        json_obj = kwargs.get("obj")
        path_expr = jsonpath_ng.parse(triple_map.reference)
        results = [r.value for r in path_expr.find(json_obj)]
        for row in results:
            if rdflib.term._is_valid_uri(row):
                return rdflib.URIRef(row)

    def __reference_handler__(self, **kwargs):
        """Internal method for handling rr:reference in triples map

        Keyword Args:

        -------------
            predicate_obj_map: SimpleNamespace
            obj: dict
            subject: rdflib.URIRef
        """
        subjects = []
        pred_obj_map = kwargs.get("predicate_obj_map")
        obj = kwargs.get("obj")
        subject = kwargs.get("subject")
        if pred_obj_map.reference is None:
            return subjects
        predicate = pred_obj_map.predicate
        ref_exp = jsonpath_ng.parse(str(pred_obj_map.refernce))
        found_objects = [r.value for r in ref_exp(obj)]
        for row in found_objects:
            self.output.append(subject, predicate, rdflib.Literal(row))
            
 
    def execute(self, triple_map, **kwargs):
        """Method executes mapping between JSON source and
        output RDF

        Args:

        -----
            triple_map: SimpleNamespace
        """
        subjects = []
        logical_src_iterator = str(triple_map.logicalSource.iterator)
        json_object = kwargs.get('obj', self.source)
        # Removes '.' as a generic iterator, replace with '@'
        if logical_src_iterator == ".":
            results = [None,]
        else:
            json_path_exp = jsonpath_ng.parse(logical_src_iterator)
            results = [r.value for r in json_path_exp.find(json_object)][0]
        for row in results:
            subject = self.generate_term(term_map=triple_map.subjectMap,
                                         **kwargs)
            for pred_obj_map in triple_map.predicateObjectMap:
                predicate = pred_obj_map.predicate
                if pred_obj_map.template is not None:
                    self.output.add((
                        subject,
                        predicate,
                        self.generate_term(term_map=pred_obj_map, **kwargs)))
                
                if pred_obj_map.parentTriplesMap is not None:
                    self.__handle_parents__(
                        parent_map=pred_obj_map.parentTriplesMap,
                        subject=subject,
                        predicate=predicate,
                        obj=row,
                        **kwargs)
                if pred_obj_map.reference is not None:
                    ref_exp = jsonpath_ng.parse(str(pred_obj_map.reference))
                    found_objects = [r.value for r in ref_exp.find(row)]
                    for obj in found_objects:
                        if rdflib.term._is_valid_uri(obj):
                            rdf_obj = rdflib.URIRef(str(obj))
                        else:
                            rdf_obj = rdflib.Literal(str(obj))
                        self.output.add((subject, predicate, ref_obj))
                if pred_obj_map.constant is not None:
                    self.output.add((subject,
                                     predicate,
                                     pred_obj_map.constant))
            subjects.append(subject)
        return subjects
            
        


    def run(self, source, **kwargs):
        """Method takes a JSON source and any keywords and transforms from
        JSON to Lean BIBFRAME 2.0 triples

        Args:

        ----
            source: str, dict
        """
        self.output = self.__graph__()
        if isinstance(source, str):
            import json
            source = json.loads(source)
        self.source = source
        super(JSONProcessor, self).run(**kwargs)
        

class XMLProcessor(Processor):
    """XML RDF Mapping Processor"""

    def __init__(self, **kwargs):
        if "rml_rules" in kwargs:
            rml_rules = kwargs.pop("rml_rules")
        super(XMLProcessor, self).__init__(rml_rules)
        if "namespaces" in kwargs:
            self.xml_ns = kwargs.pop("namespaces")
        else:
            self.xml_ns = dict()
        self.constants.update(kwargs)

    def __generate_reference__(self, triple_map, **kwargs):
        """Internal method takes a triple_map and returns the result of
        applying to XPath to the current DOM context

        Args:
        -----
            triple_map: SimpleNamespace
            element: etree.Element
        """
        element = kwargs.get("element")
        found_elements = element.findall(triple_map.reference, self.xml_ns)
        for elem in found_elements:
            #! Quick and dirty test for valid URI
            if not elem.text.startswith("http"):
                continue
            return rdflib.URIRef(elem.text)


    def __reference_handler__(self, **kwargs):
        """Internal method for handling rr:reference in triples map

        Keyword Args:

        -------------
            predicate_obj_map: SimpleNamespace
            element: etree.Element
            subject: rdflib.URIRef
        """
        subjects = []
        pred_obj_map = kwargs.get("predicate_obj_map")
        element = kwargs.get("element")
        subject = kwargs.get("subject")
        if pred_obj_map.reference is None:
            return subjects
        predicate = pred_obj_map.predicate
        found_elements = element.findall(str(pred_obj_map.reference),
                                         self.xml_ns)
        for found_elem in found_elements:
            if found_elem.text is None or len(found_elem.text) < 1:
                continue
            if pred_obj_map.constant is not None:
                self.output.add((subject,
                                 predicate,
                                 pred_obj_map.constant))
                continue
            if not hasattr(pred_obj_map, "datatype") or \
                pred_obj_map.datatype is None:
                datatype = None
            else:
                datatype = pred_obj_map.datatype
            if pred_obj_map.delimiters != []:
               subjects.extend(
                    self.__generate_delimited_objects__(
                        triple_map=pred_obj_map,
                        subject=subject,
                        predicate=predicate,
                        element=found_elem,
                        delimiters=pred_obj_map.delimiters,
                        datatype=datatype))
            else:
                object_ = self.__generate_object_term__(datatype, found_elem.text)
                self.output.add((subject, predicate, object_))
        return subjects


    def execute(self, triple_map, **kwargs):
        """Method executes mapping between source

        Args:

        -----
            triple_map: SimpleNamespace, Triple Map

        """
        subjects = []
        for element in self.source.findall(
                str(triple_map.logicalSource.iterator),
                self.xml_ns):
            subject = self.generate_term(term_map=triple_map.subjectMap,
                                         element=element,
                                         **kwargs)
            start = len(self.output)
            for row in triple_map.predicateObjectMap:
                predicate = row.predicate
                if row.template is not None:
                    self.output.add((
                        subject,
                        predicate,
                        self.generate_term(term_map=row, **kwargs)))
                if row.parentTriplesMap is not None:
                    self.__handle_parents__(
                        parent_map=row.parentTriplesMap,
                        subject=subject,
                        predicate=predicate,
                        **kwargs)
                subjects.extend(self.__reference_handler__(
                    predicate_obj_map=row,
                    element=element,
                    subject=subject))
                if row.constant is not None:
                    self.output.add((subject,
                                     predicate,
                                     row.constant))
            if start < len(self.output):
                if triple_map.subjectMap.class_ is not None:
                    self.output.add((subject,
                                     NS_MGR.rdf.type,
                                     triple_map.subjectMap.class_))
                subjects.append(subject)
        return subjects



    def run(self, xml, **kwargs):
        self.output = self.__graph__()
        if isinstance(xml, str):
            self.source = etree.XML(xml)
        else:
            self.source = xml
        super(XMLProcessor, self).run(**kwargs)


def __get_object__(binding):
    """Method takes a binding extracts value and returns rdflib
     entity

     Args:
         binding: binding row
    """
    if isinstance(binding, rdflib.term.Node):
        return binding
    elif isinstance(binding, collections.Iterable):
        for key, row in binding.items():
            if isinstance(row, (rdflib.URIRef, rdflib.Literal)):
                return row
            elif isinstance(row, dict):
                if row.get('type').startswith('uri'):
                    return rdflib.URIRef(row.get('value'))
                #elif row.get('type').startswith('literal'):
                else:
                    return rdflib.Literal(row.get('value'))
            elif isinstance(row, tuple):
                print(row)
            elif isinstance(row, str):
                if row.startswith("literal") or "xml:lang" in key:
                    continue
                return rdflib.Literal(row)

 


class SPARQLProcessor(Processor):
    """SPARQLProcessor provides a RML Processor for external SPARQL endpoints"""

    def __init__(self, **kwargs):
        if "rml_rules" in kwargs:
            rml_rules = kwargs.pop("rml_rules")
        super(SPARQLProcessor, self).__init__(rml_rules)
        __set_prefix__()
        self.triplestore_url = kwargs.get("triplestore_url")
        if self.triplestore_url is None:
            # Tries using rdflib Graph as triplestore
            self.triplestore = kwargs.get("triplestore", self.__graph__())

        # Sets defaults
        self.limit, self.offset = 5000, 0

    def __get_bindings__(self, sparql, output_format):
        """Internal method queries triplestore or remote
        sparal endpont and returns the bindings

        Args:

        ----
            sparql: String of SPARQL query
            output_format: String of type of outputform
        """
        if self.triplestore_url is None:
            result = self.triplestore.query(sparql)
            bindings = result.bindings
        else:
            result = requests.post(
                self.triplestore_url,
                data={"query": sparql,
                      "format": output_format})
            if output_format == "json":
                bindings = result.json().get("results").get("bindings")
            elif output_format == "xml":
                xml_doc = etree.XML(result.text)
                bindings = xml_doc.findall("results/bindings")
        return bindings

    def run(self, **kwargs):
        self.output = self.__graph__()
        if "limit" in kwargs:
            self.limit = kwargs.get('limit')
        if "offset" in kwargs:
            self.offset = kwargs.get('offset')
        super(SPARQLProcessor, self).run(**kwargs)

    def execute(self, triple_map, **kwargs):
        """Execute """
        subjects = []
        if NS_MGR.ql.JSON in triple_map.logicalSource.reference_formulations:
            output_format = "json"
        else:
            output_format = "xml"
        iterator = str(triple_map.logicalSource.iterator)
        if 'limit' not in kwargs:
            kwargs['limit'] = self.limit
        if 'offset' not in kwargs:
            kwargs['offset'] = self.offset
        iterator = str(triple_map.logicalSource.iterator)
        sparql = PREFIX + triple_map.logicalSource.query.format(
            **kwargs)
        bindings = self.__get_bindings__(sparql, output_format)
        #import pdb; pdb.set_trace()
        for binding in bindings:
            entity_raw = binding.get(iterator)
            if isinstance(entity_raw, (rdflib.URIRef, rdflib.BNode)):
                entity = entity_raw
            else:
                raw_value = entity_raw.get('value')
                if entity_raw.get('type').startswith('bnode'):
                    entity = rdflib.BNode(raw_value)
                else:
                    entity = rdflib.URIRef(raw_value)
            if triple_map.subjectMap.class_ is not None:
                self.output.add((entity,
                                 NS_MGR.rdf.type,
                                 triple_map.subjectMap.class_))
            for pred_obj_map in triple_map.predicateObjectMap:
                predicate = pred_obj_map.predicate
                kwargs[iterator] = entity

                if pred_obj_map.parentTriplesMap is not None:
                    self.__handle_parents__(
                        parent_map=pred_obj_map.parentTriplesMap,
                        subject=entity,
                        predicate=predicate,
                        **kwargs)
                    continue
                if pred_obj_map.reference is not None:
                    #import pdb; pdb.set_trace()
                    ref_key = str(pred_obj_map.reference)
                    if ref_key in binding:
                        object_ = __get_object__(
                            binding[ref_key])
                        self.output.add((entity, predicate, object_))
                    continue
                if pred_obj_map.constant is not None:
                    self.output.add(
                        (entity, predicate, pred_obj_map.constant))
                    continue
                sparql_query = PREFIX + pred_obj_map.query.format(
                    **kwargs)
                pre_obj_bindings = self.__get_bindings__(
                    sparql_query,
                    output_format)
                for row in pre_obj_bindings:
                    object_ = __get_object__(row)
                    self.output.add((entity, predicate, object_))
            subjects.append(entity)
        return subjects


def __set_prefix__():
    global PREFIX
    PREFIX = ""
    for row in dir(NS_MGR):
        if row.startswith("__") or row is None:
            continue
        PREFIX += "PREFIX {0}: <{1}>\n".format(row, getattr(NS_MGR, row))
    return PREFIX

PREFIX = __set_prefix__()

DEDUP_RULE = PREFIX + """
SELECT DISTINCT ?class ?bf_match
WHERE {
   ?map rr:subjectMap ?sub_map .
   ?sub_map rr:class ?class .
   ?sub_map kds:deduplicate ?bf_match
}"""


DEDUP_TRIPLESTORE = PREFIX + """
SELECT DISTINCT ?subj
WHERE {{
    ?subj rdf:type <{0}> .
    ?subj <{1}> ?name .
    FILTER(CONTAINS(?name, \"""{2}\"""))
}}"""


GET_TRIPLE_MAPS = PREFIX + """
SELECT DISTINCT ?map
WHERE {
    ?map rdf:type rr:TriplesMap .
}"""



__author__ = "Jeremy Nelson, Mike Stabile"
import datetime
import logging
import os
import pymarc
import rdflib
import requests
import sys
import unittest
import uuid

from types import SimpleNamespace

PROJECT_BASE = os.path.abspath(os.path.curdir)
sys.path.append(PROJECT_BASE)
#sys.path.append(os.path.join(PROJECT_BASE, "bibcat"))
#print(os.path.join(PROJECT_BASE, "bibcat"))
import bibcat.ingesters 
from bibcat.ingesters.ingester import Ingester, new_graph, MNAME

NS_MGR = SimpleNamespace()

NS_MGR.bf = rdflib.Namespace("http://id.loc.gov/ontologies/bibframe/")
NS_MGR.kds = rdflib.Namespace("http://knowledgelinks.io/ns/data-structures/")
NS_MGR.schema = rdflib.Namespace("http://schema.org/")
NS_MGR.owl = rdflib.Namespace(rdflib.OWL)
NS_MGR.relators = rdflib.Namespace("http://id.loc.gov/vocabulary/relators/")

bibcat.ingesters.MLOG_LVL = logging.CRITICAL
logging.getLogger("requests").setLevel(logging.CRITICAL)
logging.getLogger("urllib3").setLevel(logging.CRITICAL)

class TestAddingAdminData(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test/resource")
        self.ingester = Ingester(rules_ttl="test.ttl")
        
        
    def test_add_admin_metadata(self):
        self.ingester.add_admin_metadata(self.entity)
        subjects = [s for s in self.ingester.graph.subjects()]
        self.assertTrue(self.entity in subjects)
        self.assertEqual(len(subjects), 4)
        
    def test_type_of(self):
        sparql = """SELECT ?bnode
WHERE {{
    ?bnode a <{0}> .
}}"""
        self.ingester.add_admin_metadata(self.entity)
        results = [row for row in self.ingester.graph.query(
            sparql.format(NS_MGR.bf.GenerationProcess))]
        self.assertEqual(len(results), 1)
        self.assertTrue(isinstance(results[0][0], rdflib.BNode))

    def test_generate_date(self):
        sparql = """SELECT ?generationDate
WHERE {{
    ?bnode <{0}> ?generationDate .
}}""".format(NS_MGR.bf.generationDate)
        self.ingester.add_admin_metadata(self.entity)
        results = [r for r in self.ingester.graph.query(sparql)]
        date_literal = results[0][0]
        generate_date = datetime.datetime.strptime(
            date_literal, 
            "%Y-%m-%dT%H:%M:%S.%f")
        self.assertTrue(generate_date < datetime.datetime.utcnow())

    def test_rdf_value(self):
        sparql = """SELECT ?default
WHERE {
    ?bnode <http://www.w3.org/1999/02/22-rdf-syntax-ns#value> ?default .
}"""
        self.ingester.add_admin_metadata(self.entity)
        results = [r for r in self.ingester.graph.query(sparql)]
        message = results[0][0]
        self.assertTrue(
            message.startswith(
                "Generated by BIBCAT version"))
        

class TestAddingToTriplestore(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")

    def test_add_to_triplestore(self):
        print(self.ingester.graph.serialize(format="turtle").decode())
        self.ingester.add_to_triplestore()
   
class TestGenerateURI(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")

    def test_default(self):
        self.assertTrue(
            str(self.ingester.__generate_uri__()).startswith(
                "http"))

    def test_custom_base_url(self):
        ingester = Ingester(base_url="http://test.edu", 
            rules_ttl="test.ttl")
        self.assertTrue(
            str(ingester.__generate_uri__()).startswith(
                "http://test.edu/"))

class TestInitIngester(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")

    def test_default_base_url(self):
        self.assertIsNotNone(self.ingester.base_url)

    def test_missing_rules_ttl(self):
        self.assertRaises(ValueError, Ingester)

    def test_default_graph(self):
        self.assertEqual(
            len(self.ingester.graph),
            0)

    def test_default_rules_graph(self):
        self.assertEqual(
            len(self.ingester.rules_graph),
            0)

    def test_default_source(self):
        self.assertIsNone(self.ingester.source)

    def test_default_triplestore_url(self):
        self.assertEqual(
            self.ingester.triplestore_url,
            "http://localhost:9999/blazegraph/sparql")

class TestInitLogSetup(unittest.TestCase):

    def test_module_mname(self):
        self.assertEqual(MNAME, 
            os.path.join(
                os.path.join(bibcat.ingesters.ingester.BIBCAT_BASE, "ingesters"),
                "ingester.py"))

class TestNewExistingBNode(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")

    def test_new_bnode(self):
        result = self.ingester.new_existing_bnode(
            NS_MGR.schema.name,
            "M24510a")
        self.assertTrue(isinstance(result, rdflib.BNode)) 

class TestNewGraph(unittest.TestCase):

    def setUp(self):
        self.graph = new_graph()

    def test_new_graph(self):
        default_turtle = self.graph.serialize(format='turtle').decode()
        self.assertTrue(len(default_turtle) > 400)

class TestPopulateEntity(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")


    def test_new_instance(self):
        entity = self.ingester.populate_entity(
            NS_MGR.bf.Instance)
        self.assertEqual(
            self.ingester.graph.value(subject=entity, 
                predicate=rdflib.RDF.type),
            NS_MGR.bf.Instance)

class TestRemoveBlankNodes(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")
        self.bnode = rdflib.BNode()
        self.ingester.graph.add(
            (self.bnode, 
             rdflib.RDF.type, 
             NS_MGR.bf.Isbn))

    def test_flat_remove_blank_nodes(self):
        self.assertTrue(
            self.bnode in [s for s in self.ingester.graph.subjects()])
        self.ingester.remove_blank_nodes(self.bnode)
        self.assertFalse(
            self.bnode in [s for s in self.ingester.graph.subjects()])


class TestReplaceURIs(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")
        self.old_uri = rdflib.URIRef("http://old.bibcat.org/{}".format(
            uuid.uuid1()))
        self.work_uri = rdflib.URIRef("http://bibcat.org/{}".format(
            uuid.uuid1()))
        self.ingester.graph.add((
            self.old_uri, 
            NS_MGR.bf.instanceOf, 
            self.work_uri))
        self.new_uri = rdflib.URIRef("http://new.bibcat.org/{}".format(
            uuid.uuid1()))

    def test_simple_replace_uri(self):
        self.assertEqual(
            self.ingester.graph.value(
                predicate=NS_MGR.bf.instanceOf,
                object=self.work_uri),
            self.old_uri)
        self.ingester.replace_uris(self.old_uri, self.new_uri)
        self.assertEqual(
            self.ingester.graph.value(
                predicate=NS_MGR.bf.instanceOf,
                object=self.work_uri),
            self.new_uri)


class TestTransform(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")

    def test_default_transform(self):
        bf_instance, bf_item = self.ingester.transform()
        self.assertIsInstance(bf_instance, rdflib.URIRef)
        self.assertIsInstance(bf_item, rdflib.URIRef)


class TestUpdateDirectProperties(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")
        self.entity = rdflib.URIRef("http://example.org/test")

    def test_default_method(self):
        self.assertIsNone(
            self.ingester.update_direct_properties(
                NS_MGR.bf.Item, 
                self.entity))

    def test_basic_rule(self):
        self.ingester.rules_graph.parse(
            data="""@prefix bc: <http://knowledgelinks.io/ns/bibcat/> .
@prefix bf: <http://id.loc.gov/ontologies/bibframe/> .
@prefix m21: <http://knowledgelinks.io/ns/marc21/> .
@prefix kds: <http://knowledgelinks.io/ns/data-structures/> .
@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .

bc:m21-bf_copyrightDate a kds:PropertyLinker;
    kds:srcPropUri m21:M260__c;
    kds:destClassUri bf:Instance;
    kds:destPropUri bf:copyrightDate.""",
            format="turtle")
        self.ingester.update_direct_properties(
                NS_MGR.bf.Item, 
                self.entity)


class TestUpdateLinkedClasses(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")
        self.entity = rdflib.URIRef("http://example.org/test")

    def test_default_method(self):
        self.assertIsNone(
            self.ingester.update_linked_classes(
                NS_MGR.bf.Item, 
                self.entity))

class TestUpdateOrderedLinkedClasses(unittest.TestCase):

    def setUp(self):
        self.ingester = Ingester(rules_ttl="test.ttl")
        self.entity = rdflib.URIRef("http://example.org/test")

    def test_default_method(self):
        self.assertIsNone(
            self.ingester.update_ordered_linked_classes(
                NS_MGR.bf.Item,
                self.entity))
 


if __name__ == '__main__':
    unittest.main()



"""SPARQL Templates and direct Statements for BIBCAT Generators"""
try:
    from .generator import NS_MGR
# Failed relative import
except SystemError:
    from generator import NS_MGR

NS_MGR.bind("bf", "http://id.loc.gov/ontologies/bibframe/")
PREFIX = NS_MGR.prefix()

DELETE_COLLECTION_BNODE = PREFIX + """
DELETE {{
    ?collection ?p ?o .
    <{0}> bf:partOf ?collection
}} 
INSERT {{ }}
WHERE {{
    <{0}> bf:partOf ?collection .
    ?collection ?p ?o
    filter isBlank(?collection)
}}"""


DELETE_WORK_BNODE = PREFIX + """
DELETE {{
    ?work ?p ?o .
    <{0}> bf:instanceOf ?work
}} 
INSERT {{ }}
WHERE {{
    <{0}> bf:instanceOf ?work .
    ?work ?p ?o
    filter isBlank(?work)
}}"""

GET_AVAILABLE_COLLECTIONS = PREFIX + """
SELECT ?instance ?org ?item ?label
WHERE {
    ?instance rdf:type bf:Instance .
    ?instance bf:partOf ?collection .
    ?collection rdf:type pcdm:Collection .
    ?collection rdfs:label ?label .
    ?item bf:itemOf ?instance .
    ?item bf:heldBy ?org
    filter(isblank(?collection))
}"""

GET_AVAILABLE_INSTANCES = PREFIX + """
SELECT ?instance 
WHERE {
    ?instance rdf:type bf:Instance .
    ?instance bf:instanceOf ?work .
    filter(isblank(?work))
}"""



GET_INSTANCE_CREATOR = PREFIX + """
SELECT ?name ?creator
WHERE {{
    <{0}> <{1}> ?creator .
    OPTIONAL {{ ?creator rdfs:label ?name }} 
    OPTIONAL {{ ?creator schema:name ?name }}
    OPTIONAL {{ ?creator schema:alternativeName ?name }}
}}"""

GET_INSTANCE_TITLE = PREFIX + """
SELECT ?mainTitle ?subTitle
WHERE {{
    <{0}> bf:title ?title .
    ?title a bf:InstanceTitle .
    ?title bf:mainTitle ?mainTitle .
    OPTIONAL {{ ?title bf:subtitle ?subTitle }}
}}"""

GET_INSTANCE_WORK_BNODE_PROPS = PREFIX + """
SELECT ?pred ?obj
WHERE {{
    <{0}> bf:instanceOf ?work .
    ?work ?pred ?obj .
    filter(isblank(?work))
}}"""

FILTER_COLLECTION = PREFIX + """
SELECT ?collection ?instance
WHERE {{
    ?collection rdf:type bf:Work .
    ?collection rdf:type pcdm:Collection .
    ?collection rdfs:label ?label .
    ?collection bf:hasPart ?instance .
    <{0}> bf:itemOf ?instance .
    <{0}> bf:heldBy <{1}> 
    FILTER(isiri(?collection))
    FILTER CONTAINS("{2}", ?label)
}}"""

FILTER_WORK_CREATOR = PREFIX + """
SELECT ?work
WHERE {{
    ?work rdf:type bf:Work .
    ?work relators:cre ?creator .
    OPTIONAL {{ ?creator schema:name ?name }}
    OPTIONAL {{ ?creator schema:alternativeName ?name }}
    FILTER(isuri(?work))
    FILTER CONTAINS("{0}", ?name)
}}"""

FILTER_WORK_TITLE = PREFIX + """
SELECT ?work
WHERE {{
    ?work rdf:type bf:Work .
    ?work bf:title ?title .
    ?title ?mainTitle """ + '"""{0}""" .' + """
    FILTER(isuri(?work)) 
}}"""



import rdflib
import unittest
from bibcat import replace_iri, slugify, wikify

__author__ = "Jeremy Nelson"

class Test_replace_iri(unittest.TestCase):
    
    def setUp(self):
        self.graph = rdflib.Graph()

    def test_bnode_old_iri_exception(self):
        pass


class Test_slugify(unittest.TestCase):

    def setUp(self):
        pass

    def test_simple_name(self):
        self.assertEqual("mexico-city",
            slugify("Mexico City"))


class Test_wikify(unittest.TestCase):

    def setUp(self):
        pass

    def test_simple_name(self):
        self.assertEqual("Tokyo_Japan",
            wikify("Tokyo, Japan"))

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



import logging
import os
import rdflib
import requests
import sys
import unittest

import xml.etree.ElementTree as etree

#sys.path.append(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(os.path.abspath(os.path.curdir))
import ingesters
import ingesters.dc as dc
dc.NS_MGR.log_level = logging.CRITICAL
ingesters.MLOG_LVL = logging.CRITICAL
dc.MLOG_LVL = logging.CRITICAL
logging.getLogger("requests").setLevel(logging.CRITICAL)
logging.getLogger("urllib3").setLevel(logging.CRITICAL)

with open(os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 
            "dpl-dc-example.xml")) as raw_file:
    dc_fixure = raw_file.read()

class TestInitDCIngester(unittest.TestCase):

    def setUp(self):
        self.ingester = dc.DCIngester()

    def test_default_is_ingester(self):
        self.assertIsInstance(self.ingester,
            dc.DCIngester)

    def test_dc_xml(self):
        ingester = dc.DCIngester(source=dc_fixure)
        self.assertIsInstance(ingester,
            dc.DCIngester)


class TestDC__handle_linked_pattern__(unittest.TestCase):

    def setUp(self):
        self.ingester = dc.DCIngester(source=dc_fixure)

    def test_exists(self):
        self.assertTrue(hasattr(self.ingester, "__handle_linked_pattern__"))

    def test_no_keywords(self):
        self.assertRaises(
            AssertionError,
            self.ingester.__handle_linked_pattern__)

    def tearDown(self):
        self.ingester.graph.close()


class TestDCUpdateLinkedClasses(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test.org/entity/1")
        self.ingester = dc.DCIngester(source=dc_fixure)

    def test_default_method(self):
        self.ingester.update_linked_classes(
            ingesters.ingester.NS_MGR.bf.Item,
            self.entity)

    def tearDown(self):
        pass

class TestDCUpdateDirectProperties(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test.org/entity/1")
        self.ingester = dc.DCIngester()


    def test_default_method(self):
        self.ingester.update_direct_properties(
            ingesters.ingester.NS_MGR.bf.Instance,
            self.entity)

class TestDCUpdateOrderedLinkedClasses(unittest.TestCase):

    def setUp(self):
        self.entity = rdflib.URIRef("http://test.org/entity/1")
        self.ingester = dc.DCIngester(source=dc_fixure)
       
    def test_default_method(self):
        self.ingester.update_ordered_linked_classes(
            ingesters.ingester.NS_MGR.bf.Item,
            self.entity)


if __name__ == '__main__':
   unittest.main() 



"""Deduplicator class attempts a simple comparison of values and labels of
target RDF classes on the input RDF graph, generates new IRIs in the triplestore
"""

from types import SimpleNamespace
import rdflib
import requests

from bibcat import replace_iri, slugify
try:
    import instance.config as config
except ImportError:
    config = SimpleNamespace()
    config.BASE_URL = "https://bibcat.org/"
    config.TRIPLESTORE_URL = "http://localhost:9999/blazegraph/sparql"

__author__ = "Jeremy Nelson"

SKOS = rdflib.Namespace("http://www.w3.org/2004/02/skos/core#")

class Deduplicator(object):
    """Class de-duplicates and generates IRIs"""

    def __init__(self, **kwargs):
        self.triplestore_url = kwargs.get(
            'triplestore_url')
        self.output = None
        self.default_classes = kwargs.get("classes", [])
        self.subject_pattern = kwargs.get("subject_pattern",
            "{base_url}{class_name}/{label}")
        self.base_url = kwargs.get("base_url")

    def __get_or_mint__(self, old_iri, iri_class, label):
        """Attempts to retrieve any existing IRIs that match the label
        if not found, mints a new IRI. Returns an existing or new
        entity IRI

        Args:

        -----
            old_iri: rdflib.URIRef IRI 
            iri_class: rdflib.URIRef predicate IRI
            label: string of rdflib.Literal for the RDFS label
        """
        sparql = """prefix bf: <http://id.loc.gov/ontologies/bibframe/>
        prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#>

        SELECT DISTINCT ?entity ?label
        WHERE {{
            ?entity rdf:type <{0}> ;
            OPTIONAL {{ ?entity rdfs:label ?label . }}
            OPTIONAL {{ ?entity rdf:value ?label . }}
            FILTER(CONTAINS(?label, \"""{1}\"""))
        }}""".format(iri_class, label)
        result = requests.post(self.triplestore_url,
            data={"query": sparql,
                  "format": "json"})
        #import pdb; pdb.set_trace()
        if result.status_code > 399:
            return
        bindings = result.json().get('results').get('bindings')
        if len(bindings) > 0:
            # Use first binding
            entity_iri = rdflib.URIRef(bindings[0].get('entity').get('value'))
            existing_label = rdflib.Literal(bindings[0].get('label').get('value'))
            if existing_label != label:
                # Add label as an skos:altLabel
                self.output.add((entity_iri, 
                                 SKOS.altLabel, 
                                 label))
        else:
            class_name = str(iri_class).split("/")[-1].lower()
            # Mint new IRI based on class and slugged label
            new_url = self.subject_pattern.format(
                base_url=self.base_url,
                class_name=class_name,
                label=slugify(label))
            entity_iri = rdflib.URIRef(new_url)
            #self.output.add((entity_iri, rdflib.RDFS.label, label))
        replace_iri(self.output, old_iri, entity_iri)
        # Add old iri as owl:sameAs to entity_iri
        if isinstance(old_iri, rdflib.URIRef):
            self.output.add((entity_iri, 
                             rdflib.OWL.sameAs,
                             old_iri))
        return entity_iri


    def run(self, input_graph, rdf_classes=[]):
        """Takes a graph and deduplicates various RDF classes

        Args:

        -----
            graph: rdflib.Graph or rdflib.ConjunctiveGraph
            rdf_classes: list of RDF Classes to use in filtering
                         IRIs
        """
        self.output = input_graph
        if rdf_classes is None:
            rdf_classes = []
        all_classes = self.default_classes + rdf_classes
        for class_ in all_classes:
            for entity in self.output.subjects(
                    predicate=rdflib.RDF.type,
                    object=class_):
                label = self.output.value(subject=entity,
                                          predicate=rdflib.RDFS.label)
                if label is not None:
                    self.__get_or_mint__(entity, class_, label)
                value = self.output.value(subject=entity,
                                          predicate=rdflib.RDF.value)
                if value is not None:
                    self.__get_or_mint__(entity, class_, value)



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))


setup(
    name="bibcat",
    version="1.12.0",
    description="BIBCAT RDF Framework Application",
    author="KnowledgeLinks",
    author_email="knowledgelinks.io@gmail.com",
    license="MIT",
    url="http://knowledgelinks.io/#/products/bibcat",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Topic :: Database",
        "Topic :: Text Processing :: General",
        "Topic :: Text Processing :: Indexing"
    ],
    keywords="semantic web bibframe rdf",
    packages=find_packages(exclude=['tests']),
    install_requires=[
        'Flask',
        'jsonpath_ng',
        'lxml',
        'pymarc',
        'rdflib',
        'rdflib-jsonld',
        'requests'
    ],
    include_package_data=True,
    package_data={
        "rdf-references": ["*.ttl"],
        "rdfw-definitions": ["*.ttl"],
        "maps": ["*.ttl"]
    }
)      



"""CSV file to BIBFRAME 2.0 Linked Data"""
__author__ = "Jeremy Nelson, Mike Stabile"

import click
import csv
import logging
import rdflib

try:
    from ingesters.ingester import Ingester, new_graph, NS_MGR
    from ingesters.sparql import GET_ADDL_PROPS
except ImportError:
    from .ingester import Ingester, new_graph, NS_MGR
    from .sparql import GET_ADDL_PROPS

MLOG_LVL = logging.DEBUG
logging.basicConfig(level=logging.DEBUG)

class RowIngester(Ingester):
    """Comma-separated Value Class takes a filepath to a CSV file where the
    the first row is assumed to be the column names""" 

    def __init__(self, source=None, rules_ttl=None):
        rules = ['kds-bibcat-csv-ingestion.ttl',]
        if isinstance(rules_ttl, str):
            rules.append(rules_ttl)
        elif isinstance(rules_ttl, list):
            rules.extend(rules_ttl) 
        super(RowIngester, self).__init__(
            rules_ttl=rules,
            source=source)

    def __handle_linked_pattern__(self, **kwargs):
        """Helper takes an entity, a rule made up of the field's key, and
        extracts and saves the destination property to the destination 
        class.

        Keyword args:
            entity(rdflib.URIRef): Entity's URI
            rule(str): String literal 
            destination_class(rdflib.URIRef): Destination class
            destination_property(rdflib.URIRef): Destination property
        """
        entity = kwargs.get("entity")
        rule = kwargs.get("rule")
        destination_class = kwargs.get("destination_class")
        destination_property = kwargs.get("destination_property")
        target_property = kwargs.get("target_property")
        target_subject = kwargs.get("target_subject")
        delimiter = self.rules_graph.value(
            subject=target_subject,
            predicate=NS_MGR.kds.delimiterProp)
        value = self.source.get(str(rule))
        if len(str(value).strip()) < 1:
            return
        if delimiter is not None:
            for row in value.split(str(delimiter)):
                filtered_value = row.strip()
                bnode_class = self.new_existing_bnode(
                    target_property,
                    target_subject)
                self.graph.add(
                    (entity,
                     target_property,
                     bnode_class))
                self.graph.add(
                    (bnode_class,
                     rdflib.RDF.type,
                     destination_class))
                self.graph.add(
                    (bnode_class,
                     destination_property,
                     rdflib.Literal(filtered_value)))
                for pred, obj in self.rules_graph.query(
                        GET_ADDL_PROPS.format(target_subject)):
                    self.graph.add((bnode_class, pred, obj))
        else:
            bnode_class = self.new_existing_bnode(
                 target_property,
                 target_subject)
            self.graph.add(
                    (entity,
                     target_property,
                     bnode_class))
            self.graph.add(
                (bnode_class,
                 rdflib.RDF.type,
                 destination_class))
            self.graph.add(
                (bnode_class,
                 destination_property,
                 rdflib.Literal(value.strip())))
            for pred, obj in self.rules_graph.query(
                    GET_ADDL_PROPS.format(target_subject)):
                self.graph.add((bnode_class, pred, obj))
           

    def __handle_pattern__(self, **kwargs):
        """Helper takes an entity, rule, BIBFRAME class, rdfs:label 
        and extracts and saves the destination property to the destination
        class.

        Keyword args:
            entity(rdflib.URIRef): Entity's URI
            rule(str): Key for row
            destination_class(rdflib.URIRef): Destination class
            destination_property(rdflib.URIRef): Destination property
            target_property(rdflib.URIRef): Target range in final class
            target_subject(rdflib.URIRef): Rule subject URI in rules graph
        """
        entity = kwargs.get("entity")
        rule = kwargs.get("rule")
        destination_class = kwargs.get("destination_class")
        destination_property = kwargs.get("destination_property")
        if isinstance(destination_property, rdflib.BNode):
            for pred, obj in self.rules_graph.predicate_objects(
                    subject=destination_property):
                self.graph.add((entity, pred, obj))
        elif rule is not None:
            value = self.source.get(rule)
            bnode_class = self.new_existing_bnode(
                target_property,
                target_subject)
            self.graph.add(
                (bnode_class,
                 rdflib.RDF.type,
                 destination_class))
            self.graph.add(
                (bnode_class,
                 destination_property,
                 rdflib.Literal(value)))

    def transform(self, row=None):
        """Method calls Ingester.transform method from parent on either
        the current instance's source or on a passed in row from CSV
        Row reader.

        Args:
            row(dict): Row, default is None
        """
        if row is not None:
            self.source = row
            self.graph = new_graph()
        super(RowIngester, self).transform()
        self.deduplicate_agents(
            NS_MGR.schema.name,
            NS_MGR.bf.Agent,
            None)
